"""The trigger commands module.

These commands are used in the following models:
AWG5K, AWG5KC, AWG7K, AWG7KC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - TRIGger:SEQuence:IMMediate
    - TRIGger:SEQuence:IMPedance <impedance>
    - TRIGger:SEQuence:IMPedance?
    - TRIGger:SEQuence:LEVel <NR3>
    - TRIGger:SEQuence:LEVel?
    - TRIGger:SEQuence:MODE <trigger_type>
    - TRIGger:SEQuence:MODE?
    - TRIGger:SEQuence:POLarity {POSitive|NEGative}
    - TRIGger:SEQuence:POLarity?
    - TRIGger:SEQuence:SLOPe {POSitive|NEGative}
    - TRIGger:SEQuence:SLOPe?
    - TRIGger:SEQuence:SOURce {INTernal|EXTernal}
    - TRIGger:SEQuence:SOURce?
    - TRIGger:SEQuence:TIMer <NR3>
    - TRIGger:SEQuence:TIMer?
    - TRIGger:SEQuence:WVALue {FIRSt|LAST}
    - TRIGger:SEQuence:WVALue?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class TriggerSequenceWvalue(SCPICmdWrite, SCPICmdRead):
    """The ``TRIGger:SEQuence:WVALue`` command.

    Description:
        - This command and query sets or returns the output data position of a waveform while the
          instrument is in the waiting-for-trigger state. This is valid only when Run Mode is
          Triggered or Gated.

    Usage:
        - Using the ``.query()`` method will send the ``TRIGger:SEQuence:WVALue?`` query.
        - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence:WVALue?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:WVALue value``
          command.

    SCPI Syntax:
        ```
        - TRIGger:SEQuence:WVALue {FIRSt|LAST}
        - TRIGger:SEQuence:WVALue?
        ```

    Info:
        - ``FIRSt`` specifies the first value of the waveform as the output level.
        - ``LAST`` specifies the last value of the waveform as the output level.
    """


class TriggerSequenceTimer(SCPICmdWrite, SCPICmdRead):
    """The ``TRIGger:SEQuence:TIMer`` command.

    Description:
        - This command and query sets or returns the internal trigger rate (trigger interval).

    Usage:
        - Using the ``.query()`` method will send the ``TRIGger:SEQuence:TIMer?`` query.
        - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence:TIMer?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:TIMer value`` command.

    SCPI Syntax:
        ```
        - TRIGger:SEQuence:TIMer <NR3>
        - TRIGger:SEQuence:TIMer?
        ```

    Info:
        - ``<NR3>``
    """


class TriggerSequenceSource(SCPICmdWrite, SCPICmdRead):
    """The ``TRIGger:SEQuence:SOURce`` command.

    Description:
        - This command and query sets or returns the trigger source.

    Usage:
        - Using the ``.query()`` method will send the ``TRIGger:SEQuence:SOURce?`` query.
        - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence:SOURce?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:SOURce value``
          command.

    SCPI Syntax:
        ```
        - TRIGger:SEQuence:SOURce {INTernal|EXTernal}
        - TRIGger:SEQuence:SOURce?
        ```

    Info:
        - ``INTernal`` selects internal clock as the trigger source.
        - ``EXTernal`` selects external clock as the trigger source.
    """


class TriggerSequenceSlope(SCPICmdWrite, SCPICmdRead):
    """The ``TRIGger:SEQuence:SLOPe`` command.

    Description:
        - This command sets or queries the slope of trigger signal.

    Usage:
        - Using the ``.query()`` method will send the ``TRIGger:SEQuence:SLOPe?`` query.
        - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence:SLOPe?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:SLOPe value`` command.

    SCPI Syntax:
        ```
        - TRIGger:SEQuence:SLOPe {POSitive|NEGative}
        - TRIGger:SEQuence:SLOPe?
        ```

    Info:
        - ``POSitive`` indicates that the event occurs on the rising edge of the external trigger
          signal.
        - ``NEGative`` indicates that the event occurs on the falling edge of the external trigger
          signal.
    """


class TriggerSequencePolarity(SCPICmdWrite, SCPICmdRead):
    """The ``TRIGger:SEQuence:POLarity`` command.

    Description:
        - This command and query sets or returns the trigger input polarity. It is used to set
          polarity in gated mode.

    Usage:
        - Using the ``.query()`` method will send the ``TRIGger:SEQuence:POLarity?`` query.
        - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence:POLarity?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:POLarity value``
          command.

    SCPI Syntax:
        ```
        - TRIGger:SEQuence:POLarity {POSitive|NEGative}
        - TRIGger:SEQuence:POLarity?
        ```

    Info:
        - ``POSitive`` means the gate signal is activated when the external trigger signal is
          greater (more Positive) than the trigger level.
        - ``NEGative`` means the gate signal is activated when the external trigger signal is less
          (more Negative) than the trigger level.
    """


class TriggerSequenceMode(SCPICmdWrite, SCPICmdRead):
    r"""The ``TRIGger:SEQuence:MODE`` command.

    Description:
        - (AWG7000B and AWG7000C Series only) This command and query sets or returns the trigger
          timing. It is used in the Triggered or Sequence mode. Trigger timing can be set when the
          external trigger source is selected.

    Usage:
        - Using the ``.query()`` method will send the ``TRIGger:SEQuence:MODE?`` query.
        - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence:MODE?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:MODE value`` command.

    SCPI Syntax:
        ```
        - TRIGger:SEQuence:MODE <trigger_type>
        - TRIGger:SEQuence:MODE?
        ```

    Info:
        - ``<trigger_type`` >::={SYNChronous\|ASYNchronous}.
    """


class TriggerSequenceLevel(SCPICmdWrite, SCPICmdRead):
    """The ``TRIGger:SEQuence:LEVel`` command.

    Description:
        - This command and query sets or returns the trigger input level (threshold).

    Usage:
        - Using the ``.query()`` method will send the ``TRIGger:SEQuence:LEVel?`` query.
        - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence:LEVel?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:LEVel value`` command.

    SCPI Syntax:
        ```
        - TRIGger:SEQuence:LEVel <NR3>
        - TRIGger:SEQuence:LEVel?
        ```

    Info:
        - ``<NR3>``
    """


class TriggerSequenceImpedance(SCPICmdWrite, SCPICmdRead):
    """The ``TRIGger:SEQuence:IMPedance`` command.

    Description:
        - This command and query sets or returns the trigger impedance. It applies only to the
          external trigger.

    Usage:
        - Using the ``.query()`` method will send the ``TRIGger:SEQuence:IMPedance?`` query.
        - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence:IMPedance?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:IMPedance value``
          command.

    SCPI Syntax:
        ```
        - TRIGger:SEQuence:IMPedance <impedance>
        - TRIGger:SEQuence:IMPedance?
        ```

    Info:
        - ``<impedance>`` ::=<NR3> the value will be 50 and 1000.
    """


class TriggerSequenceImmediate(SCPICmdWriteNoArguments):
    """The ``TRIGger:SEQuence:IMMediate`` command.

    Description:
        - This command forces a trigger event to occur.

    Usage:
        - Using the ``.write()`` method will send the ``TRIGger:SEQuence:IMMediate`` command.

    SCPI Syntax:
        ```
        - TRIGger:SEQuence:IMMediate
        ```
    """


#  pylint: disable=too-many-instance-attributes
class TriggerSequence(SCPICmdRead):
    """The ``TRIGger:SEQuence`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``TRIGger:SEQuence?`` query.
        - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.impedance``: The ``TRIGger:SEQuence:IMPedance`` command.
        - ``.level``: The ``TRIGger:SEQuence:LEVel`` command.
        - ``.mode``: The ``TRIGger:SEQuence:MODE`` command.
        - ``.polarity``: The ``TRIGger:SEQuence:POLarity`` command.
        - ``.slope``: The ``TRIGger:SEQuence:SLOPe`` command.
        - ``.source``: The ``TRIGger:SEQuence:SOURce`` command.
        - ``.timer``: The ``TRIGger:SEQuence:TIMer`` command.
        - ``.wvalue``: The ``TRIGger:SEQuence:WVALue`` command.
        - ``.immediate``: The ``TRIGger:SEQuence:IMMediate`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._impedance = TriggerSequenceImpedance(device, f"{self._cmd_syntax}:IMPedance")
        self._level = TriggerSequenceLevel(device, f"{self._cmd_syntax}:LEVel")
        self._mode = TriggerSequenceMode(device, f"{self._cmd_syntax}:MODE")
        self._polarity = TriggerSequencePolarity(device, f"{self._cmd_syntax}:POLarity")
        self._slope = TriggerSequenceSlope(device, f"{self._cmd_syntax}:SLOPe")
        self._source = TriggerSequenceSource(device, f"{self._cmd_syntax}:SOURce")
        self._timer = TriggerSequenceTimer(device, f"{self._cmd_syntax}:TIMer")
        self._wvalue = TriggerSequenceWvalue(device, f"{self._cmd_syntax}:WVALue")
        self._immediate = TriggerSequenceImmediate(device, f"{self._cmd_syntax}:IMMediate")

    @property
    def impedance(self) -> TriggerSequenceImpedance:
        """Return the ``TRIGger:SEQuence:IMPedance`` command.

        Description:
            - This command and query sets or returns the trigger impedance. It applies only to the
              external trigger.

        Usage:
            - Using the ``.query()`` method will send the ``TRIGger:SEQuence:IMPedance?`` query.
            - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence:IMPedance?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:IMPedance value``
              command.

        SCPI Syntax:
            ```
            - TRIGger:SEQuence:IMPedance <impedance>
            - TRIGger:SEQuence:IMPedance?
            ```

        Info:
            - ``<impedance>`` ::=<NR3> the value will be 50 and 1000.
        """
        return self._impedance

    @property
    def level(self) -> TriggerSequenceLevel:
        """Return the ``TRIGger:SEQuence:LEVel`` command.

        Description:
            - This command and query sets or returns the trigger input level (threshold).

        Usage:
            - Using the ``.query()`` method will send the ``TRIGger:SEQuence:LEVel?`` query.
            - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence:LEVel?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:LEVel value``
              command.

        SCPI Syntax:
            ```
            - TRIGger:SEQuence:LEVel <NR3>
            - TRIGger:SEQuence:LEVel?
            ```

        Info:
            - ``<NR3>``
        """
        return self._level

    @property
    def mode(self) -> TriggerSequenceMode:
        r"""Return the ``TRIGger:SEQuence:MODE`` command.

        Description:
            - (AWG7000B and AWG7000C Series only) This command and query sets or returns the trigger
              timing. It is used in the Triggered or Sequence mode. Trigger timing can be set when
              the external trigger source is selected.

        Usage:
            - Using the ``.query()`` method will send the ``TRIGger:SEQuence:MODE?`` query.
            - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence:MODE?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:MODE value``
              command.

        SCPI Syntax:
            ```
            - TRIGger:SEQuence:MODE <trigger_type>
            - TRIGger:SEQuence:MODE?
            ```

        Info:
            - ``<trigger_type`` >::={SYNChronous\|ASYNchronous}.
        """
        return self._mode

    @property
    def polarity(self) -> TriggerSequencePolarity:
        """Return the ``TRIGger:SEQuence:POLarity`` command.

        Description:
            - This command and query sets or returns the trigger input polarity. It is used to set
              polarity in gated mode.

        Usage:
            - Using the ``.query()`` method will send the ``TRIGger:SEQuence:POLarity?`` query.
            - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence:POLarity?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:POLarity value``
              command.

        SCPI Syntax:
            ```
            - TRIGger:SEQuence:POLarity {POSitive|NEGative}
            - TRIGger:SEQuence:POLarity?
            ```

        Info:
            - ``POSitive`` means the gate signal is activated when the external trigger signal is
              greater (more Positive) than the trigger level.
            - ``NEGative`` means the gate signal is activated when the external trigger signal is
              less (more Negative) than the trigger level.
        """
        return self._polarity

    @property
    def slope(self) -> TriggerSequenceSlope:
        """Return the ``TRIGger:SEQuence:SLOPe`` command.

        Description:
            - This command sets or queries the slope of trigger signal.

        Usage:
            - Using the ``.query()`` method will send the ``TRIGger:SEQuence:SLOPe?`` query.
            - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence:SLOPe?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:SLOPe value``
              command.

        SCPI Syntax:
            ```
            - TRIGger:SEQuence:SLOPe {POSitive|NEGative}
            - TRIGger:SEQuence:SLOPe?
            ```

        Info:
            - ``POSitive`` indicates that the event occurs on the rising edge of the external
              trigger signal.
            - ``NEGative`` indicates that the event occurs on the falling edge of the external
              trigger signal.
        """
        return self._slope

    @property
    def source(self) -> TriggerSequenceSource:
        """Return the ``TRIGger:SEQuence:SOURce`` command.

        Description:
            - This command and query sets or returns the trigger source.

        Usage:
            - Using the ``.query()`` method will send the ``TRIGger:SEQuence:SOURce?`` query.
            - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence:SOURce?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:SOURce value``
              command.

        SCPI Syntax:
            ```
            - TRIGger:SEQuence:SOURce {INTernal|EXTernal}
            - TRIGger:SEQuence:SOURce?
            ```

        Info:
            - ``INTernal`` selects internal clock as the trigger source.
            - ``EXTernal`` selects external clock as the trigger source.
        """
        return self._source

    @property
    def timer(self) -> TriggerSequenceTimer:
        """Return the ``TRIGger:SEQuence:TIMer`` command.

        Description:
            - This command and query sets or returns the internal trigger rate (trigger interval).

        Usage:
            - Using the ``.query()`` method will send the ``TRIGger:SEQuence:TIMer?`` query.
            - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence:TIMer?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:TIMer value``
              command.

        SCPI Syntax:
            ```
            - TRIGger:SEQuence:TIMer <NR3>
            - TRIGger:SEQuence:TIMer?
            ```

        Info:
            - ``<NR3>``
        """
        return self._timer

    @property
    def wvalue(self) -> TriggerSequenceWvalue:
        """Return the ``TRIGger:SEQuence:WVALue`` command.

        Description:
            - This command and query sets or returns the output data position of a waveform while
              the instrument is in the waiting-for-trigger state. This is valid only when Run Mode
              is Triggered or Gated.

        Usage:
            - Using the ``.query()`` method will send the ``TRIGger:SEQuence:WVALue?`` query.
            - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence:WVALue?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``TRIGger:SEQuence:WVALue value``
              command.

        SCPI Syntax:
            ```
            - TRIGger:SEQuence:WVALue {FIRSt|LAST}
            - TRIGger:SEQuence:WVALue?
            ```

        Info:
            - ``FIRSt`` specifies the first value of the waveform as the output level.
            - ``LAST`` specifies the last value of the waveform as the output level.
        """
        return self._wvalue

    @property
    def immediate(self) -> TriggerSequenceImmediate:
        """Return the ``TRIGger:SEQuence:IMMediate`` command.

        Description:
            - This command forces a trigger event to occur.

        Usage:
            - Using the ``.write()`` method will send the ``TRIGger:SEQuence:IMMediate`` command.

        SCPI Syntax:
            ```
            - TRIGger:SEQuence:IMMediate
            ```
        """
        return self._immediate


class Trigger(SCPICmdRead):
    """The ``TRIGger`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``TRIGger?`` query.
        - Using the ``.verify(value)`` method will send the ``TRIGger?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.sequence``: The ``TRIGger:SEQuence`` command tree.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "TRIGger") -> None:
        super().__init__(device, cmd_syntax)
        self._sequence = TriggerSequence(device, f"{self._cmd_syntax}:SEQuence")

    @property
    def sequence(self) -> TriggerSequence:
        """Return the ``TRIGger:SEQuence`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``TRIGger:SEQuence?`` query.
            - Using the ``.verify(value)`` method will send the ``TRIGger:SEQuence?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.impedance``: The ``TRIGger:SEQuence:IMPedance`` command.
            - ``.level``: The ``TRIGger:SEQuence:LEVel`` command.
            - ``.mode``: The ``TRIGger:SEQuence:MODE`` command.
            - ``.polarity``: The ``TRIGger:SEQuence:POLarity`` command.
            - ``.slope``: The ``TRIGger:SEQuence:SLOPe`` command.
            - ``.source``: The ``TRIGger:SEQuence:SOURce`` command.
            - ``.timer``: The ``TRIGger:SEQuence:TIMer`` command.
            - ``.wvalue``: The ``TRIGger:SEQuence:WVALue`` command.
            - ``.immediate``: The ``TRIGger:SEQuence:IMMediate`` command.
        """
        return self._sequence



# pylint: disable=line-too-long
"""The fpanel commands module.

These commands are used in the following models:
LPD6, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - FPAnel:PRESS {AUTOset|BUS|CH<x>|CLEAR|CURsor|DEFaultsetup|FASTAcq|FORCetrig|GPKNOB<x>|HIGHRES|HORZPOS|HORZScale|MATh|NEXt|PREv|REF|RUNSTop|SETTO50|SINGleseq|TOUCHSCReen|TRIGMode|TRIGSlope|USER|VERTPOS|VERTSCALE|ZOOM}
    - FPAnel:TURN {GPKNOB<x>|HORZPOS|HORZScale|PANKNOB|TRIGLevel|VERTPOS|VERTSCALE|ZOOM} [,<NR1>]
    ```
"""  # noqa: E501

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class FpanelTurn(SCPICmdWrite):
    """The ``FPAnel:TURN`` command.

    Description:
        - This command is used to emulate a knob turn. The optional NR1 specifies the number of
          clicks where negative values indicate counter clockwise. If not specified, the default of
          1 click is used indicating the knob is turned clockwise 1 click.

    Usage:
        - Using the ``.write(value)`` method will send the ``FPAnel:TURN value`` command.

    SCPI Syntax:
        ```
        - FPAnel:TURN {GPKNOB<x>|HORZPOS|HORZScale|PANKNOB|TRIGLevel|VERTPOS|VERTSCALE|ZOOM} [,<NR1>]
        ```

    Info:
        - ``GPKNOB1`` emulates a Multipurpose A knob turn.
        - ``GPKNOB2`` emulates a Multipurpose B knob turn.
        - ``HORZPOS`` emulates a Horizontal knob turn.
        - ``HORZScale`` emulates a Horizontal Scale knob turn.
        - ``PANKNOB`` emulates a Pan knob turn.
        - ``TRIGLevel`` emulates a Trigger Level knob turn.
        - ``VERTPOS`` emulates a Vertical Position knob turn.
        - ``VERTSCALE`` emulates a Vertical Scale knob turn.
        - ``ZOOM`` emulates a Zoom knob turn.
        - ``<NR1>`` is the number of clicks to turn the knob.
    """  # noqa: E501


class FpanelPress(SCPICmdWrite):
    """The ``FPAnel:PRESS`` command.

    Description:
        - This command is used to emulate a button press. When used with knob enumerations, this
          command pushes the knob. Use the ``FPAnel:TURN`` command to emulate knob turns.

    Usage:
        - Using the ``.write(value)`` method will send the ``FPAnel:PRESS value`` command.

    SCPI Syntax:
        ```
        - FPAnel:PRESS {AUTOset|BUS|CH<x>|CLEAR|CURsor|DEFaultsetup|FASTAcq|FORCetrig|GPKNOB<x>|HIGHRES|HORZPOS|HORZScale|MATh|NEXt|PREv|REF|RUNSTop|SETTO50|SINGleseq|TOUCHSCReen|TRIGMode|TRIGSlope|USER|VERTPOS|VERTSCALE|ZOOM}
        ```

    Info:
        - ``AUTOset`` emulates a Autoset button press.
        - ``BUS`` emulates a Bus button press.
        - ``CH<x>`` emulates a Channel button press. The channel number is specified by x.
        - ``CLEAR`` emulates a Clear button press.
        - ``CURsor`` emulates a Cursor button press.
        - ``DEFaultsetup`` emulates a Default Setup button press.
        - ``FASTAcq`` emulates a Fast Acq button press.
        - ``FORCetrig`` emulates a Force trigger button press.
        - ``GPKNOB1`` emulates a Multipurpose A knob press.
        - ``GPKNOB2`` emulates a Multipurpose B knob press.
        - ``HIGHRES`` emulates a High Res button press.
        - ``HORZPOS`` emulates a Horizontal knob press.
        - ``HORZScale`` emulates a Horizontal Scale knob press.
        - ``MATh`` emulates a Math button press.
        - ``NEXt`` emulates a Next button press.
        - ``PREv`` emulates a Previous button press.
        - ``REF`` emulates a Ref button press.
        - ``RUNSTop`` emulates a Run/Stop button press.
        - ``SETTO50`` emulates a Set to 50% threshold button press.
        - ``SINGleseq`` emulates a Single/Seq button press.
        - ``TOUCHSCReen`` emulates a Touch Screen button press.
        - ``TRIGMode`` emulates a Trigger Mode button press.
        - ``TRIGSlope`` emulates a Trigger Slope button press.
        - ``USER`` emulates a User button press.
        - ``VERTPOS`` emulates a Vertical Position knob press.
        - ``VERTSCALE`` emulates a Vertical Scale knob press.
        - ``ZOOM`` emulates a Zoom button press.
    """  # noqa: E501


class Fpanel(SCPICmdRead):
    """The ``FPAnel`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``FPAnel?`` query.
        - Using the ``.verify(value)`` method will send the ``FPAnel?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.press``: The ``FPAnel:PRESS`` command.
        - ``.turn``: The ``FPAnel:TURN`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "FPAnel") -> None:
        super().__init__(device, cmd_syntax)
        self._press = FpanelPress(device, f"{self._cmd_syntax}:PRESS")
        self._turn = FpanelTurn(device, f"{self._cmd_syntax}:TURN")

    @property
    def press(self) -> FpanelPress:
        """Return the ``FPAnel:PRESS`` command.

        Description:
            - This command is used to emulate a button press. When used with knob enumerations, this
              command pushes the knob. Use the ``FPAnel:TURN`` command to emulate knob turns.

        Usage:
            - Using the ``.write(value)`` method will send the ``FPAnel:PRESS value`` command.

        SCPI Syntax:
            ```
            - FPAnel:PRESS {AUTOset|BUS|CH<x>|CLEAR|CURsor|DEFaultsetup|FASTAcq|FORCetrig|GPKNOB<x>|HIGHRES|HORZPOS|HORZScale|MATh|NEXt|PREv|REF|RUNSTop|SETTO50|SINGleseq|TOUCHSCReen|TRIGMode|TRIGSlope|USER|VERTPOS|VERTSCALE|ZOOM}
            ```

        Info:
            - ``AUTOset`` emulates a Autoset button press.
            - ``BUS`` emulates a Bus button press.
            - ``CH<x>`` emulates a Channel button press. The channel number is specified by x.
            - ``CLEAR`` emulates a Clear button press.
            - ``CURsor`` emulates a Cursor button press.
            - ``DEFaultsetup`` emulates a Default Setup button press.
            - ``FASTAcq`` emulates a Fast Acq button press.
            - ``FORCetrig`` emulates a Force trigger button press.
            - ``GPKNOB1`` emulates a Multipurpose A knob press.
            - ``GPKNOB2`` emulates a Multipurpose B knob press.
            - ``HIGHRES`` emulates a High Res button press.
            - ``HORZPOS`` emulates a Horizontal knob press.
            - ``HORZScale`` emulates a Horizontal Scale knob press.
            - ``MATh`` emulates a Math button press.
            - ``NEXt`` emulates a Next button press.
            - ``PREv`` emulates a Previous button press.
            - ``REF`` emulates a Ref button press.
            - ``RUNSTop`` emulates a Run/Stop button press.
            - ``SETTO50`` emulates a Set to 50% threshold button press.
            - ``SINGleseq`` emulates a Single/Seq button press.
            - ``TOUCHSCReen`` emulates a Touch Screen button press.
            - ``TRIGMode`` emulates a Trigger Mode button press.
            - ``TRIGSlope`` emulates a Trigger Slope button press.
            - ``USER`` emulates a User button press.
            - ``VERTPOS`` emulates a Vertical Position knob press.
            - ``VERTSCALE`` emulates a Vertical Scale knob press.
            - ``ZOOM`` emulates a Zoom button press.
        """  # noqa: E501
        return self._press

    @property
    def turn(self) -> FpanelTurn:
        """Return the ``FPAnel:TURN`` command.

        Description:
            - This command is used to emulate a knob turn. The optional NR1 specifies the number of
              clicks where negative values indicate counter clockwise. If not specified, the default
              of 1 click is used indicating the knob is turned clockwise 1 click.

        Usage:
            - Using the ``.write(value)`` method will send the ``FPAnel:TURN value`` command.

        SCPI Syntax:
            ```
            - FPAnel:TURN {GPKNOB<x>|HORZPOS|HORZScale|PANKNOB|TRIGLevel|VERTPOS|VERTSCALE|ZOOM} [,<NR1>]
            ```

        Info:
            - ``GPKNOB1`` emulates a Multipurpose A knob turn.
            - ``GPKNOB2`` emulates a Multipurpose B knob turn.
            - ``HORZPOS`` emulates a Horizontal knob turn.
            - ``HORZScale`` emulates a Horizontal Scale knob turn.
            - ``PANKNOB`` emulates a Pan knob turn.
            - ``TRIGLevel`` emulates a Trigger Level knob turn.
            - ``VERTPOS`` emulates a Vertical Position knob turn.
            - ``VERTSCALE`` emulates a Vertical Scale knob turn.
            - ``ZOOM`` emulates a Zoom knob turn.
            - ``<NR1>`` is the number of clicks to turn the knob.
        """  # noqa: E501
        return self._turn



"""The ethernet commands module.

These commands are used in the following models:
DPO7AX, LPD6, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - ETHERnet:DHCPbootp {ON|OFF}
    - ETHERnet:DHCPbootp?
    - ETHERnet:DNS:IPADDress <QString>
    - ETHERnet:DNS:IPADDress?
    - ETHERnet:DOMAINname <QString>
    - ETHERnet:DOMAINname?
    - ETHERnet:ENET:ADDress?
    - ETHERnet:GATEWay:IPADDress <QString>
    - ETHERnet:GATEWay:IPADDress?
    - ETHERnet:IPADDress <QString>
    - ETHERnet:IPADDress?
    - ETHERnet:LXI:LAN:RESET
    - ETHERnet:LXI:LAN:SERVICENAMe <QString>
    - ETHERnet:LXI:LAN:SERVICENAMe?
    - ETHERnet:LXI:LAN:STATus?
    - ETHERnet:NAME <QString>
    - ETHERnet:NAME?
    - ETHERnet:NETWORKCONFig {AUTOmatic|MANual}
    - ETHERnet:NETWORKCONFig?
    - ETHERnet:PING EXECute
    - ETHERnet:PING:STATus?
    - ETHERnet:SUBNETMask <QString>
    - ETHERnet:SUBNETMask?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite, SCPICmdWriteNoArguments

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class EthernetSubnetmask(SCPICmdWrite, SCPICmdRead):
    """The ``ETHERnet:SUBNETMask`` command.

    Description:
        - This command sets or queries the instrument subnet mask value.

    Usage:
        - Using the ``.query()`` method will send the ``ETHERnet:SUBNETMask?`` query.
        - Using the ``.verify(value)`` method will send the ``ETHERnet:SUBNETMask?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ETHERnet:SUBNETMask value`` command.

    SCPI Syntax:
        ```
        - ETHERnet:SUBNETMask <QString>
        - ETHERnet:SUBNETMask?
        ```

    Info:
        - ``<QString>`` is the subnet mask value, enclosed in quotes.
    """

    _WRAP_ARG_WITH_QUOTES = True


class EthernetPingStatus(SCPICmdRead):
    """The ``ETHERnet:PING:STATus`` command.

    Description:
        - Returns the results of sending the ``ETHERnet:PING`` command to ping the gateway IP
          address.

    Usage:
        - Using the ``.query()`` method will send the ``ETHERnet:PING:STATus?`` query.
        - Using the ``.verify(value)`` method will send the ``ETHERnet:PING:STATus?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - ETHERnet:PING:STATus?
        ```
    """


class EthernetPing(SCPICmdWrite, SCPICmdRead):
    """The ``ETHERnet:PING`` command.

    Description:
        - Sends a ping packet to the instrument gateway and sets the status accordingly.

    Usage:
        - Using the ``.write(value)`` method will send the ``ETHERnet:PING value`` command.

    SCPI Syntax:
        ```
        - ETHERnet:PING EXECute
        ```

    Properties:
        - ``.status``: The ``ETHERnet:PING:STATus`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._status = EthernetPingStatus(device, f"{self._cmd_syntax}:STATus")

    @property
    def status(self) -> EthernetPingStatus:
        """Return the ``ETHERnet:PING:STATus`` command.

        Description:
            - Returns the results of sending the ``ETHERnet:PING`` command to ping the gateway IP
              address.

        Usage:
            - Using the ``.query()`` method will send the ``ETHERnet:PING:STATus?`` query.
            - Using the ``.verify(value)`` method will send the ``ETHERnet:PING:STATus?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - ETHERnet:PING:STATus?
            ```
        """
        return self._status


class EthernetNetworkconfig(SCPICmdWrite, SCPICmdRead):
    """The ``ETHERnet:NETWORKCONFig`` command.

    Description:
        - This command specifies the Ethernet network configuration setting.

    Usage:
        - Using the ``.query()`` method will send the ``ETHERnet:NETWORKCONFig?`` query.
        - Using the ``.verify(value)`` method will send the ``ETHERnet:NETWORKCONFig?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ETHERnet:NETWORKCONFig value`` command.

    SCPI Syntax:
        ```
        - ETHERnet:NETWORKCONFig {AUTOmatic|MANual}
        - ETHERnet:NETWORKCONFig?
        ```

    Info:
        - ``AUTOmatic`` specifies that the instrument's IP address, subnet mask and gateway settings
          will be received from a DHCP server on the local network.
        - ``MANual`` specifies that the Ethernet settings will be configured manually. When setting
          to manual mode, the following commands must also be
          specified;``ETHERnet:IPADDress``,``ETHERnet:GATEWay:IPADDress``, and
          ``ETHERnet:SUBNETMask``. All the settings must be sent in a single string in order for the
          oscilloscope to save the setting. If sending only ``ETHERnet:NETWORKCONFig MANual``, the
          user interface will reset back to auto mode. Since the Apply Changes button in the user
          interface does not have an associated command, sending the whole string will be required
          in order to update the static IP address when using manual network configuration mode.
    """


class EthernetName(SCPICmdWrite, SCPICmdRead):
    """The ``ETHERnet:NAME`` command.

    Description:
        - This command sets or queries the instrument Ethernet hostname assigned to the instrument.

    Usage:
        - Using the ``.query()`` method will send the ``ETHERnet:NAME?`` query.
        - Using the ``.verify(value)`` method will send the ``ETHERnet:NAME?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ETHERnet:NAME value`` command.

    SCPI Syntax:
        ```
        - ETHERnet:NAME <QString>
        - ETHERnet:NAME?
        ```

    Info:
        - ``<QString>`` is the network name assigned to the instrument, enclosed in quotes.
    """

    _WRAP_ARG_WITH_QUOTES = True


class EthernetLxiLanStatus(SCPICmdRead):
    """The ``ETHERnet:LXI:LAN:STATus`` command.

    Description:
        - This query returns the LXI network status: one of OK, FAULT, or IDENTIFY. IDENTIFY
          indicates that the device identify mode is enabled.

    Usage:
        - Using the ``.query()`` method will send the ``ETHERnet:LXI:LAN:STATus?`` query.
        - Using the ``.verify(value)`` method will send the ``ETHERnet:LXI:LAN:STATus?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - ETHERnet:LXI:LAN:STATus?
        ```
    """


class EthernetLxiLanServicename(SCPICmdWrite, SCPICmdRead):
    """The ``ETHERnet:LXI:LAN:SERVICENAMe`` command.

    Description:
        - This command sets or queries the service name used for the LXI interface.

    Usage:
        - Using the ``.query()`` method will send the ``ETHERnet:LXI:LAN:SERVICENAMe?`` query.
        - Using the ``.verify(value)`` method will send the ``ETHERnet:LXI:LAN:SERVICENAMe?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ETHERnet:LXI:LAN:SERVICENAMe value``
          command.

    SCPI Syntax:
        ```
        - ETHERnet:LXI:LAN:SERVICENAMe <QString>
        - ETHERnet:LXI:LAN:SERVICENAMe?
        ```

    Info:
        - ``<QString>`` is a quoted string of up to 64 characters that specifies the mDNS service
          name used for the LXI interface.
    """

    _WRAP_ARG_WITH_QUOTES = True


class EthernetLxiLanReset(SCPICmdWriteNoArguments):
    """The ``ETHERnet:LXI:LAN:RESET`` command.

    Description:
        - This command resets the LXI local area network.

    Usage:
        - Using the ``.write()`` method will send the ``ETHERnet:LXI:LAN:RESET`` command.

    SCPI Syntax:
        ```
        - ETHERnet:LXI:LAN:RESET
        ```
    """


class EthernetLxiLan(SCPICmdRead):
    """The ``ETHERnet:LXI:LAN`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``ETHERnet:LXI:LAN?`` query.
        - Using the ``.verify(value)`` method will send the ``ETHERnet:LXI:LAN?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.reset``: The ``ETHERnet:LXI:LAN:RESET`` command.
        - ``.servicename``: The ``ETHERnet:LXI:LAN:SERVICENAMe`` command.
        - ``.status``: The ``ETHERnet:LXI:LAN:STATus`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._reset = EthernetLxiLanReset(device, f"{self._cmd_syntax}:RESET")
        self._servicename = EthernetLxiLanServicename(device, f"{self._cmd_syntax}:SERVICENAMe")
        self._status = EthernetLxiLanStatus(device, f"{self._cmd_syntax}:STATus")

    @property
    def reset(self) -> EthernetLxiLanReset:
        """Return the ``ETHERnet:LXI:LAN:RESET`` command.

        Description:
            - This command resets the LXI local area network.

        Usage:
            - Using the ``.write()`` method will send the ``ETHERnet:LXI:LAN:RESET`` command.

        SCPI Syntax:
            ```
            - ETHERnet:LXI:LAN:RESET
            ```
        """
        return self._reset

    @property
    def servicename(self) -> EthernetLxiLanServicename:
        """Return the ``ETHERnet:LXI:LAN:SERVICENAMe`` command.

        Description:
            - This command sets or queries the service name used for the LXI interface.

        Usage:
            - Using the ``.query()`` method will send the ``ETHERnet:LXI:LAN:SERVICENAMe?`` query.
            - Using the ``.verify(value)`` method will send the ``ETHERnet:LXI:LAN:SERVICENAMe?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``ETHERnet:LXI:LAN:SERVICENAMe value`` command.

        SCPI Syntax:
            ```
            - ETHERnet:LXI:LAN:SERVICENAMe <QString>
            - ETHERnet:LXI:LAN:SERVICENAMe?
            ```

        Info:
            - ``<QString>`` is a quoted string of up to 64 characters that specifies the mDNS
              service name used for the LXI interface.
        """
        return self._servicename

    @property
    def status(self) -> EthernetLxiLanStatus:
        """Return the ``ETHERnet:LXI:LAN:STATus`` command.

        Description:
            - This query returns the LXI network status: one of OK, FAULT, or IDENTIFY. IDENTIFY
              indicates that the device identify mode is enabled.

        Usage:
            - Using the ``.query()`` method will send the ``ETHERnet:LXI:LAN:STATus?`` query.
            - Using the ``.verify(value)`` method will send the ``ETHERnet:LXI:LAN:STATus?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - ETHERnet:LXI:LAN:STATus?
            ```
        """
        return self._status


class EthernetLxi(SCPICmdRead):
    """The ``ETHERnet:LXI`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``ETHERnet:LXI?`` query.
        - Using the ``.verify(value)`` method will send the ``ETHERnet:LXI?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.lan``: The ``ETHERnet:LXI:LAN`` command tree.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._lan = EthernetLxiLan(device, f"{self._cmd_syntax}:LAN")

    @property
    def lan(self) -> EthernetLxiLan:
        """Return the ``ETHERnet:LXI:LAN`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``ETHERnet:LXI:LAN?`` query.
            - Using the ``.verify(value)`` method will send the ``ETHERnet:LXI:LAN?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.reset``: The ``ETHERnet:LXI:LAN:RESET`` command.
            - ``.servicename``: The ``ETHERnet:LXI:LAN:SERVICENAMe`` command.
            - ``.status``: The ``ETHERnet:LXI:LAN:STATus`` command.
        """
        return self._lan


class EthernetIpaddress(SCPICmdWrite, SCPICmdRead):
    """The ``ETHERnet:IPADDress`` command.

    Description:
        - This command sets the IP address assigned to the instrument.

    Usage:
        - Using the ``.query()`` method will send the ``ETHERnet:IPADDress?`` query.
        - Using the ``.verify(value)`` method will send the ``ETHERnet:IPADDress?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ETHERnet:IPADDress value`` command.

    SCPI Syntax:
        ```
        - ETHERnet:IPADDress <QString>
        - ETHERnet:IPADDress?
        ```

    Info:
        - ``<QString>`` is a standard IP address value, enclosed in quotes.
    """

    _WRAP_ARG_WITH_QUOTES = True


class EthernetGatewayIpaddress(SCPICmdWrite, SCPICmdRead):
    """The ``ETHERnet:GATEWay:IPADDress`` command.

    Description:
        - This command specifies the network gateway IP address.

    Usage:
        - Using the ``.query()`` method will send the ``ETHERnet:GATEWay:IPADDress?`` query.
        - Using the ``.verify(value)`` method will send the ``ETHERnet:GATEWay:IPADDress?`` query
          and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ETHERnet:GATEWay:IPADDress value``
          command.

    SCPI Syntax:
        ```
        - ETHERnet:GATEWay:IPADDress <QString>
        - ETHERnet:GATEWay:IPADDress?
        ```

    Info:
        - ``<QString>`` is a standard IP address value, enclosed in quotes.
    """

    _WRAP_ARG_WITH_QUOTES = True


class EthernetGateway(SCPICmdRead):
    """The ``ETHERnet:GATEWay`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``ETHERnet:GATEWay?`` query.
        - Using the ``.verify(value)`` method will send the ``ETHERnet:GATEWay?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.ipaddress``: The ``ETHERnet:GATEWay:IPADDress`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._ipaddress = EthernetGatewayIpaddress(device, f"{self._cmd_syntax}:IPADDress")

    @property
    def ipaddress(self) -> EthernetGatewayIpaddress:
        """Return the ``ETHERnet:GATEWay:IPADDress`` command.

        Description:
            - This command specifies the network gateway IP address.

        Usage:
            - Using the ``.query()`` method will send the ``ETHERnet:GATEWay:IPADDress?`` query.
            - Using the ``.verify(value)`` method will send the ``ETHERnet:GATEWay:IPADDress?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ETHERnet:GATEWay:IPADDress value``
              command.

        SCPI Syntax:
            ```
            - ETHERnet:GATEWay:IPADDress <QString>
            - ETHERnet:GATEWay:IPADDress?
            ```

        Info:
            - ``<QString>`` is a standard IP address value, enclosed in quotes.
        """
        return self._ipaddress


class EthernetEnetAddress(SCPICmdRead):
    """The ``ETHERnet:ENET:ADDress`` command.

    Description:
        - Returns the Ethernet address (MAC address) value assigned to the instrument. This is
          assigned at the factory and can not be changed.

    Usage:
        - Using the ``.query()`` method will send the ``ETHERnet:ENET:ADDress?`` query.
        - Using the ``.verify(value)`` method will send the ``ETHERnet:ENET:ADDress?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - ETHERnet:ENET:ADDress?
        ```
    """


class EthernetEnet(SCPICmdRead):
    """The ``ETHERnet:ENET`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``ETHERnet:ENET?`` query.
        - Using the ``.verify(value)`` method will send the ``ETHERnet:ENET?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.address``: The ``ETHERnet:ENET:ADDress`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._address = EthernetEnetAddress(device, f"{self._cmd_syntax}:ADDress")

    @property
    def address(self) -> EthernetEnetAddress:
        """Return the ``ETHERnet:ENET:ADDress`` command.

        Description:
            - Returns the Ethernet address (MAC address) value assigned to the instrument. This is
              assigned at the factory and can not be changed.

        Usage:
            - Using the ``.query()`` method will send the ``ETHERnet:ENET:ADDress?`` query.
            - Using the ``.verify(value)`` method will send the ``ETHERnet:ENET:ADDress?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - ETHERnet:ENET:ADDress?
            ```
        """
        return self._address


class EthernetDomainname(SCPICmdWrite, SCPICmdRead):
    """The ``ETHERnet:DOMAINname`` command.

    Description:
        - This command specifies the network domain name.

    Usage:
        - Using the ``.query()`` method will send the ``ETHERnet:DOMAINname?`` query.
        - Using the ``.verify(value)`` method will send the ``ETHERnet:DOMAINname?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ETHERnet:DOMAINname value`` command.

    SCPI Syntax:
        ```
        - ETHERnet:DOMAINname <QString>
        - ETHERnet:DOMAINname?
        ```

    Info:
        - ``<QString>`` is the network domain name, enclosed in quotes.
    """

    _WRAP_ARG_WITH_QUOTES = True


class EthernetDnsIpaddress(SCPICmdWrite, SCPICmdRead):
    """The ``ETHERnet:DNS:IPADDress`` command.

    Description:
        - This command specifies the network Domain Name Server (DNS) IP address.

    Usage:
        - Using the ``.query()`` method will send the ``ETHERnet:DNS:IPADDress?`` query.
        - Using the ``.verify(value)`` method will send the ``ETHERnet:DNS:IPADDress?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ETHERnet:DNS:IPADDress value`` command.

    SCPI Syntax:
        ```
        - ETHERnet:DNS:IPADDress <QString>
        - ETHERnet:DNS:IPADDress?
        ```

    Info:
        - ``<QString>`` is a standard IP address value, enclosed in quotes.
    """

    _WRAP_ARG_WITH_QUOTES = True


class EthernetDns(SCPICmdRead):
    """The ``ETHERnet:DNS`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``ETHERnet:DNS?`` query.
        - Using the ``.verify(value)`` method will send the ``ETHERnet:DNS?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.ipaddress``: The ``ETHERnet:DNS:IPADDress`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._ipaddress = EthernetDnsIpaddress(device, f"{self._cmd_syntax}:IPADDress")

    @property
    def ipaddress(self) -> EthernetDnsIpaddress:
        """Return the ``ETHERnet:DNS:IPADDress`` command.

        Description:
            - This command specifies the network Domain Name Server (DNS) IP address.

        Usage:
            - Using the ``.query()`` method will send the ``ETHERnet:DNS:IPADDress?`` query.
            - Using the ``.verify(value)`` method will send the ``ETHERnet:DNS:IPADDress?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ETHERnet:DNS:IPADDress value``
              command.

        SCPI Syntax:
            ```
            - ETHERnet:DNS:IPADDress <QString>
            - ETHERnet:DNS:IPADDress?
            ```

        Info:
            - ``<QString>`` is a standard IP address value, enclosed in quotes.
        """
        return self._ipaddress


class EthernetDhcpbootp(SCPICmdWrite, SCPICmdRead):
    """The ``ETHERnet:DHCPbootp`` command.

    Description:
        - This command sets the network configuration method to DHCP (that is ON) or static IP
          address (that is OFF).

    Usage:
        - Using the ``.query()`` method will send the ``ETHERnet:DHCPbootp?`` query.
        - Using the ``.verify(value)`` method will send the ``ETHERnet:DHCPbootp?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``ETHERnet:DHCPbootp value`` command.

    SCPI Syntax:
        ```
        - ETHERnet:DHCPbootp {ON|OFF}
        - ETHERnet:DHCPbootp?
        ```

    Info:
        - ``ON`` enables the instrument to search the network for a DHCP server in order to
          automatically assign a dynamic IP address to the instrument.
        - ``OFF`` disables the instrument to search the network for a DHCP server.
    """


#  pylint: disable=too-many-instance-attributes
class Ethernet(SCPICmdRead):
    """The ``ETHERnet`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``ETHERnet?`` query.
        - Using the ``.verify(value)`` method will send the ``ETHERnet?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.dhcpbootp``: The ``ETHERnet:DHCPbootp`` command.
        - ``.dns``: The ``ETHERnet:DNS`` command tree.
        - ``.domainname``: The ``ETHERnet:DOMAINname`` command.
        - ``.enet``: The ``ETHERnet:ENET`` command tree.
        - ``.gateway``: The ``ETHERnet:GATEWay`` command tree.
        - ``.ipaddress``: The ``ETHERnet:IPADDress`` command.
        - ``.lxi``: The ``ETHERnet:LXI`` command tree.
        - ``.name``: The ``ETHERnet:NAME`` command.
        - ``.networkconfig``: The ``ETHERnet:NETWORKCONFig`` command.
        - ``.ping``: The ``ETHERnet:PING`` command.
        - ``.subnetmask``: The ``ETHERnet:SUBNETMask`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "ETHERnet") -> None:
        super().__init__(device, cmd_syntax)
        self._dhcpbootp = EthernetDhcpbootp(device, f"{self._cmd_syntax}:DHCPbootp")
        self._dns = EthernetDns(device, f"{self._cmd_syntax}:DNS")
        self._domainname = EthernetDomainname(device, f"{self._cmd_syntax}:DOMAINname")
        self._enet = EthernetEnet(device, f"{self._cmd_syntax}:ENET")
        self._gateway = EthernetGateway(device, f"{self._cmd_syntax}:GATEWay")
        self._ipaddress = EthernetIpaddress(device, f"{self._cmd_syntax}:IPADDress")
        self._lxi = EthernetLxi(device, f"{self._cmd_syntax}:LXI")
        self._name = EthernetName(device, f"{self._cmd_syntax}:NAME")
        self._networkconfig = EthernetNetworkconfig(device, f"{self._cmd_syntax}:NETWORKCONFig")
        self._ping = EthernetPing(device, f"{self._cmd_syntax}:PING")
        self._subnetmask = EthernetSubnetmask(device, f"{self._cmd_syntax}:SUBNETMask")

    @property
    def dhcpbootp(self) -> EthernetDhcpbootp:
        """Return the ``ETHERnet:DHCPbootp`` command.

        Description:
            - This command sets the network configuration method to DHCP (that is ON) or static IP
              address (that is OFF).

        Usage:
            - Using the ``.query()`` method will send the ``ETHERnet:DHCPbootp?`` query.
            - Using the ``.verify(value)`` method will send the ``ETHERnet:DHCPbootp?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ETHERnet:DHCPbootp value`` command.

        SCPI Syntax:
            ```
            - ETHERnet:DHCPbootp {ON|OFF}
            - ETHERnet:DHCPbootp?
            ```

        Info:
            - ``ON`` enables the instrument to search the network for a DHCP server in order to
              automatically assign a dynamic IP address to the instrument.
            - ``OFF`` disables the instrument to search the network for a DHCP server.
        """
        return self._dhcpbootp

    @property
    def dns(self) -> EthernetDns:
        """Return the ``ETHERnet:DNS`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``ETHERnet:DNS?`` query.
            - Using the ``.verify(value)`` method will send the ``ETHERnet:DNS?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.ipaddress``: The ``ETHERnet:DNS:IPADDress`` command.
        """
        return self._dns

    @property
    def domainname(self) -> EthernetDomainname:
        """Return the ``ETHERnet:DOMAINname`` command.

        Description:
            - This command specifies the network domain name.

        Usage:
            - Using the ``.query()`` method will send the ``ETHERnet:DOMAINname?`` query.
            - Using the ``.verify(value)`` method will send the ``ETHERnet:DOMAINname?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ETHERnet:DOMAINname value``
              command.

        SCPI Syntax:
            ```
            - ETHERnet:DOMAINname <QString>
            - ETHERnet:DOMAINname?
            ```

        Info:
            - ``<QString>`` is the network domain name, enclosed in quotes.
        """
        return self._domainname

    @property
    def enet(self) -> EthernetEnet:
        """Return the ``ETHERnet:ENET`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``ETHERnet:ENET?`` query.
            - Using the ``.verify(value)`` method will send the ``ETHERnet:ENET?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.address``: The ``ETHERnet:ENET:ADDress`` command.
        """
        return self._enet

    @property
    def gateway(self) -> EthernetGateway:
        """Return the ``ETHERnet:GATEWay`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``ETHERnet:GATEWay?`` query.
            - Using the ``.verify(value)`` method will send the ``ETHERnet:GATEWay?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.ipaddress``: The ``ETHERnet:GATEWay:IPADDress`` command.
        """
        return self._gateway

    @property
    def ipaddress(self) -> EthernetIpaddress:
        """Return the ``ETHERnet:IPADDress`` command.

        Description:
            - This command sets the IP address assigned to the instrument.

        Usage:
            - Using the ``.query()`` method will send the ``ETHERnet:IPADDress?`` query.
            - Using the ``.verify(value)`` method will send the ``ETHERnet:IPADDress?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ETHERnet:IPADDress value`` command.

        SCPI Syntax:
            ```
            - ETHERnet:IPADDress <QString>
            - ETHERnet:IPADDress?
            ```

        Info:
            - ``<QString>`` is a standard IP address value, enclosed in quotes.
        """
        return self._ipaddress

    @property
    def lxi(self) -> EthernetLxi:
        """Return the ``ETHERnet:LXI`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``ETHERnet:LXI?`` query.
            - Using the ``.verify(value)`` method will send the ``ETHERnet:LXI?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.lan``: The ``ETHERnet:LXI:LAN`` command tree.
        """
        return self._lxi

    @property
    def name(self) -> EthernetName:
        """Return the ``ETHERnet:NAME`` command.

        Description:
            - This command sets or queries the instrument Ethernet hostname assigned to the
              instrument.

        Usage:
            - Using the ``.query()`` method will send the ``ETHERnet:NAME?`` query.
            - Using the ``.verify(value)`` method will send the ``ETHERnet:NAME?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ETHERnet:NAME value`` command.

        SCPI Syntax:
            ```
            - ETHERnet:NAME <QString>
            - ETHERnet:NAME?
            ```

        Info:
            - ``<QString>`` is the network name assigned to the instrument, enclosed in quotes.
        """
        return self._name

    @property
    def networkconfig(self) -> EthernetNetworkconfig:
        """Return the ``ETHERnet:NETWORKCONFig`` command.

        Description:
            - This command specifies the Ethernet network configuration setting.

        Usage:
            - Using the ``.query()`` method will send the ``ETHERnet:NETWORKCONFig?`` query.
            - Using the ``.verify(value)`` method will send the ``ETHERnet:NETWORKCONFig?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ETHERnet:NETWORKCONFig value``
              command.

        SCPI Syntax:
            ```
            - ETHERnet:NETWORKCONFig {AUTOmatic|MANual}
            - ETHERnet:NETWORKCONFig?
            ```

        Info:
            - ``AUTOmatic`` specifies that the instrument's IP address, subnet mask and gateway
              settings will be received from a DHCP server on the local network.
            - ``MANual`` specifies that the Ethernet settings will be configured manually. When
              setting to manual mode, the following commands must also be
              specified;``ETHERnet:IPADDress``,``ETHERnet:GATEWay:IPADDress``, and
              ``ETHERnet:SUBNETMask``. All the settings must be sent in a single string in order for
              the oscilloscope to save the setting. If sending only
              ``ETHERnet:NETWORKCONFig MANual``, the user interface will reset back to auto mode.
              Since the Apply Changes button in the user interface does not have an associated
              command, sending the whole string will be required in order to update the static IP
              address when using manual network configuration mode.
        """
        return self._networkconfig

    @property
    def ping(self) -> EthernetPing:
        """Return the ``ETHERnet:PING`` command.

        Description:
            - Sends a ping packet to the instrument gateway and sets the status accordingly.

        Usage:
            - Using the ``.write(value)`` method will send the ``ETHERnet:PING value`` command.

        SCPI Syntax:
            ```
            - ETHERnet:PING EXECute
            ```

        Sub-properties:
            - ``.status``: The ``ETHERnet:PING:STATus`` command.
        """
        return self._ping

    @property
    def subnetmask(self) -> EthernetSubnetmask:
        """Return the ``ETHERnet:SUBNETMask`` command.

        Description:
            - This command sets or queries the instrument subnet mask value.

        Usage:
            - Using the ``.query()`` method will send the ``ETHERnet:SUBNETMask?`` query.
            - Using the ``.verify(value)`` method will send the ``ETHERnet:SUBNETMask?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``ETHERnet:SUBNETMask value``
              command.

        SCPI Syntax:
            ```
            - ETHERnet:SUBNETMask <QString>
            - ETHERnet:SUBNETMask?
            ```

        Info:
            - ``<QString>`` is the subnet mask value, enclosed in quotes.
        """
        return self._subnetmask



"""The status_and_error commands module.

These commands are used in the following models:
AWG5200, AWG5K, AWG5KC, AWG70KA, AWG70KB, AWG7K, AWG7KC, DPO2K, DPO2KB, DPO4K, DPO4KB, DPO5K,
DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7AX, DPO7K, DPO7KC, DSA70KC, DSA70KD, LPD6, MDO3,
MDO3K, MDO4K, MDO4KB, MDO4KC, MSO2, MSO2K, MSO2KB, MSO4, MSO4B, MSO4K, MSO4KB, MSO5, MSO5B, MSO5K,
MSO5KB, MSO5LP, MSO6, MSO6B, MSO70KC, MSO70KDX, TekScopePC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - *SRE <NR1>
    - *SRE?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Sre(SCPICmdWrite, SCPICmdRead):
    """The ``*SRE`` command.

    Description:
        - The ``*SRE`` (Service Request Enable) command sets and queries the bits in the Service
          Request Enable Register. For more information, refer to Registers.

    Usage:
        - Using the ``.query()`` method will send the ``*SRE?`` query.
        - Using the ``.verify(value)`` method will send the ``*SRE?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``*SRE value`` command.

    SCPI Syntax:
        ```
        - *SRE <NR1>
        - *SRE?
        ```

    Info:
        - ``<NR1>`` is a value in the range from 0 through 255. The binary bits of the SRER are set
          according to this value. Using an out-of-range value causes an execution error. The
          power-on default for SRER is 0 if ``*PSC`` is 1. If ``*PSC`` is 0, the SRER maintains the
          previous power cycle value through the current power cycle.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "*SRE") -> None:
        super().__init__(device, cmd_syntax)



"""Base source channel module."""

from abc import ABC, abstractmethod
from typing import Optional

from tm_devices.driver_mixins.device_control import PIControl
from tm_devices.driver_mixins.shared_implementations._extension_mixin import (
    _ExtendableMixin,  # pyright: ignore[reportPrivateUsage]
)


class BaseSourceChannel(_ExtendableMixin, ABC):
    """Base source channel."""

    def __init__(self, pi_control: PIControl, channel_name: str) -> None:
        """Create a source channel.

        Args:
            pi_control: A PI device.
            channel_name: The channel name for the source channel.
        """
        self._name = channel_name
        self._num = None
        if channel_num := "".join(filter(str.isdigit, channel_name)):
            self._num = int(channel_num)
        self._pi_control = pi_control

    @property
    def name(self) -> str:
        """Return the channel name."""
        return self._name

    @property
    def num(self) -> Optional[int]:
        """Return the channel number."""
        return self._num

    @abstractmethod
    def set_amplitude(self, value: float, absolute_tolerance: float = 0) -> None:
        """Set the amplitude on the source channel.

        Args:
            value: The amplitude value to set.
            absolute_tolerance: The acceptable difference between two floating point values.
        """
        raise NotImplementedError

    @abstractmethod
    def set_offset(self, value: float, absolute_tolerance: float = 0) -> None:
        """Set the offset on the source channel.

        Args:
            value: The offset value to set.
            absolute_tolerance: The acceptable difference between two floating point values.
        """
        raise NotImplementedError

    @abstractmethod
    def set_state(self, value: int) -> None:
        """Set the output state to ON/OFF (1/0) on the source channel.

        Args:
            value: The output state.
        """
        raise NotImplementedError



"""The cq commands module.

These commands are used in the following models:
DPO5K, DPO5KB, DPO70KC, DPO70KD, DPO70KDX, DPO70KSX, DPO7K, DPO7KC, DSA70KC, DSA70KD, MSO5K, MSO5KB,
MSO70KC, MSO70KDX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - CQ<x>:THRESHold <NR3>
    - CQ<x>:THRESHold?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite, ValidatedDynamicNumberCmd

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class CqItemThreshold(SCPICmdWrite, SCPICmdRead):
    """The ``CQ<x>:THRESHold`` command.

    Description:
        - This command sets or queries the comparable threshold for converting the clock/qualifier
          signal to its digital form, where x is zero.

    Usage:
        - Using the ``.query()`` method will send the ``CQ<x>:THRESHold?`` query.
        - Using the ``.verify(value)`` method will send the ``CQ<x>:THRESHold?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``CQ<x>:THRESHold value`` command.

    SCPI Syntax:
        ```
        - CQ<x>:THRESHold <NR3>
        - CQ<x>:THRESHold?
        ```

    Info:
        - ``<NR3>`` specifies the channel/qualifier threshold in volts.
    """


class CqItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``CQ<x>`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``CQ<x>?`` query.
        - Using the ``.verify(value)`` method will send the ``CQ<x>?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.threshold``: The ``CQ<x>:THRESHold`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "CQ<x>") -> None:
        super().__init__(device, cmd_syntax)
        self._threshold = CqItemThreshold(device, f"{self._cmd_syntax}:THRESHold")

    @property
    def threshold(self) -> CqItemThreshold:
        """Return the ``CQ<x>:THRESHold`` command.

        Description:
            - This command sets or queries the comparable threshold for converting the
              clock/qualifier signal to its digital form, where x is zero.

        Usage:
            - Using the ``.query()`` method will send the ``CQ<x>:THRESHold?`` query.
            - Using the ``.verify(value)`` method will send the ``CQ<x>:THRESHold?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``CQ<x>:THRESHold value`` command.

        SCPI Syntax:
            ```
            - CQ<x>:THRESHold <NR3>
            - CQ<x>:THRESHold?
            ```

        Info:
            - ``<NR3>`` specifies the channel/qualifier threshold in volts.
        """
        return self._threshold



"""The counter commands module.

These commands are used in the following models:
DPO70KSX

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - COUnter <REFerence|CLEAR|RESet>
    - COUnter:DURation <nr1>
    - COUnter:DURation?
    - COUnter:GAIn <nr2>
    - COUnter:GAIn?
    - COUnter:LOGAction <CLEAR>
    - COUnter:LOGNumber <nr1>
    - COUnter:LOGNumber?
    - COUnter:LOGTable <OFF|ON>
    - COUnter:LOGTable?
    - COUnter:PREscaler <OFF|ON>
    - COUnter:PREscaler?
    - COUnter:REFerence <nr3>
    - COUnter:REFerence?
    - COUnter:RESULTs:AVGmean?
    - COUnter:RESULTs:DEViation?
    - COUnter:RESULTs:MAXimum?
    - COUnter:RESULTs:MINimum?
    - COUnter:RESULTs:NUMber?
    - COUnter:RESULTs:VALue?
    - COUnter:RESULTs?
    - COUnter:SHOWLog <OFF|ON>
    - COUnter:SHOWLog?
    - COUnter:SHOWMeasurement <OFF|ON>
    - COUnter:SHOWMeasurement?
    - COUnter:STAte {ON|OFF|<NR1>}
    - COUnter:STAte?
    - COUnter:TYPe <TIME|FREQuency>
    - COUnter:TYPe?
    - COUnter:UPDate <AUTO|TIME|NUMBER>
    - COUnter:UPDate?
    - COUnter:VIEW <FREQuency|PERiod>
    - COUnter:VIEW?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class CounterView(SCPICmdWrite, SCPICmdRead):
    """The ``COUnter:VIEW`` command.

    Description:
        - Sets or queries the View of the Counter Trigger Source Frequency Application Data. When
          the ``COUnter:TYPe`` is FREQuency, the data may be viewed as Frequency (Hz) or Time
          (seconds). This command does not apply to the A-B Time Interval Application
          (``COUnter:TYPE TIME``). DPO70000SX Series only. This command is optional.

    Usage:
        - Using the ``.query()`` method will send the ``COUnter:VIEW?`` query.
        - Using the ``.verify(value)`` method will send the ``COUnter:VIEW?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``COUnter:VIEW value`` command.

    SCPI Syntax:
        ```
        - COUnter:VIEW <FREQuency|PERiod>
        - COUnter:VIEW?
        ```

    Info:
        - ``FREQuency`` displays the count in Hertz.
        - ``PERiod`` displays the count in seconds.
    """


class CounterUpdate(SCPICmdWrite, SCPICmdRead):
    """The ``COUnter:UPDate`` command.

    Description:
        - Sets or queries the parameter that controls the schedule for updating the Log Table of the
          Trigger Source Frequency or A-B Trigger Time Interval Application Data. The schedule for
          logging the data may be set to occur automatically, after a given time interval has
          elapsed, or after a given number of acquisitions have occurred. The default is AUTO.
          DPO70000SX Series only. This command is optional.

    Usage:
        - Using the ``.query()`` method will send the ``COUnter:UPDate?`` query.
        - Using the ``.verify(value)`` method will send the ``COUnter:UPDate?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``COUnter:UPDate value`` command.

    SCPI Syntax:
        ```
        - COUnter:UPDate <AUTO|TIME|NUMBER>
        - COUnter:UPDate?
        ```

    Info:
        - ``AUTO`` sets the data logging to automatic.
        - ``TIME`` sets the data logging to occur after a given time interval has elapsed.
        - ``NUMBER`` sets the data logging to occur after a given number of acquisitions have
          occurred.
    """


class CounterType(SCPICmdWrite, SCPICmdRead):
    """The ``COUnter:TYPe`` command.

    Description:
        - Sets or queries the Counter Application type as Trigger Source Frequency or A-B Trigger
          Time Interval. The Trigger Source Frequency Application is commonly referred to as
          Frequency Counting, and measures the frequency of the trigger source over a large number
          of events to obtain a very accurate result. The A-B Trigger Time Interval Application
          measures the A-Event to B-Event in an A->B Sequence Trigger, over a single A->B Trigger
          Sequence. DPO70000SX Series only. This command is optional.

    Usage:
        - Using the ``.query()`` method will send the ``COUnter:TYPe?`` query.
        - Using the ``.verify(value)`` method will send the ``COUnter:TYPe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``COUnter:TYPe value`` command.

    SCPI Syntax:
        ```
        - COUnter:TYPe <TIME|FREQuency>
        - COUnter:TYPe?
        ```

    Info:
        - ``TIME`` sets the application type to frequency.
        - ``FREQuency`` sets the application type to time interval. The default value is Frequency.
    """


class CounterState(SCPICmdWrite, SCPICmdRead):
    """The ``COUnter:STAte`` command.

    Description:
        - Enables the Trigger Source Frequency or A-B Trigger Time Interval application, depending
          on the value of the ``COUnter:TYPe`` parameter. This command sets or queries the Counter
          State enumerated value (OFF/ON) used to enable/disable the display of frequency or time
          information in the graticule area or in the Results Table. The Trigger Source Frequency
          Application uses trigger hardware, rather than the acquired waveform, to measure the
          frequency of the signal on the trigger source to a very high precision. Best results are
          obtained when the External Reference is tied between the instrument and the source of the
          trigger signal. As a result of using the trigger hardware, the measurement update rate is
          very fast, 1/3 second by default. An A-Event Edge to B-Event Edge A->B Sequence must be
          used. Statistics are derived from the measured values: Minimum, Maximum, Average (mean),
          and Standard Deviation. The number of measurements made so far is also included. Note that
          long record lengths are NOT needed for this application, because the measurements are made
          from the trigger hardware, not from the acquired waveform. A Reference value may be taken
          by command or entered manually. When the Reference is non-zero, the measured value and the
          derived statistics are shown or queried as delta values from the Reference. The A-B
          Trigger Time Interval Application also uses trigger hardware to measure the time between
          the A-Event and the B-Event in an A-B Sequence trigger. The A- and B-Events may be any
          combination of trigger types: Edge, Glitch, Width, etc. A Reference value may be taken by
          command or entered manually. When the Reference value is non-zero, the measured value is
          displayed or queried as a delta from this Reference. For example, if the ``COUnter:TYPe``
          parameter is set to FREQuency, the ``COUnter:STAte ON`` command causes the display of the
          frequency of the triggered channel signal, which is assumed to be an infinitely repeating
          clock. Along with the frequency value, the minimum, maximum, average (mean), and standard
          deviation of the frequency are also displayed. Finally, a reference value is displayed.
          The reference is 0.0 by default, but may be captured as the last measured value at any
          time using the COUnter REF command. When the reference value is nonzero, the other values
          are deltas from the reference value. The displayed information is typical of products
          generally referred to by the industry as 'requency Counters. At the user's discretion, the
          ``COUnter:VIEW`` <PERiod | FREQuency> command can be used to view the information as the
          Frequency (Hz) or Period (seconds). The default view is Frequency. As a second example, if
          the ``COUnter:TYPe`` parameter is set to TIME, the ``COUnter:STAte ON`` command causes the
          display of the Time interval from the Trigger A-Event to the Trigger B-Event. These events
          can be from any trigger types in an A->B Trigger Sequence. In this case only the Time and
          the Reference value are displayed. The COUnter REF command takes the Reference value as
          before, but in this case the units relate to Time (seconds). DPO70000SX Series only. This
          command is optional.

    Usage:
        - Using the ``.query()`` method will send the ``COUnter:STAte?`` query.
        - Using the ``.verify(value)`` method will send the ``COUnter:STAte?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``COUnter:STAte value`` command.

    SCPI Syntax:
        ```
        - COUnter:STAte {ON|OFF|<NR1>}
        - COUnter:STAte?
        ```

    Info:
        - ``<NR1>`` , if 0 disables the counter function, if 1 enables the counter function.
        - ``OFF`` disables the counter function. The default value is OFF.
        - ``ON`` enables the counter function.
    """


class CounterShowmeasurement(SCPICmdWrite, SCPICmdRead):
    """The ``COUnter:SHOWMeasurement`` command.

    Description:
        - Sets or queries the parameter that controls the display of the Trigger Source Frequency or
          A-B Trigger Time Interval Application Data. The display of the data in the graticule area
          may be turned OFF or back ON as desired. A possible reason for turning it OFF is when the
          Log Table is enabled to record the data. DPO70000SX Series only. This command is optional.

    Usage:
        - Using the ``.query()`` method will send the ``COUnter:SHOWMeasurement?`` query.
        - Using the ``.verify(value)`` method will send the ``COUnter:SHOWMeasurement?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``COUnter:SHOWMeasurement value``
          command.

    SCPI Syntax:
        ```
        - COUnter:SHOWMeasurement <OFF|ON>
        - COUnter:SHOWMeasurement?
        ```

    Info:
        - ``OFF`` disables the display of the frequency or time interval data.
        - ``ON`` enables the display of the frequency or time interval data. The default is ON.
    """


class CounterShowlog(SCPICmdWrite, SCPICmdRead):
    """The ``COUnter:SHOWLog`` command.

    Description:
        - Sets or queries the parameter that controls the display of the Trigger Source Frequency or
          A-B Trigger Time Interval Application Data. The display of the data in the graticule area
          may be turned OFF or back ON as desired. A possible reason for turning it OFF is when the
          Log Table is enabled to record the data. DPO70000SX Series only. This command is optional.

    Usage:
        - Using the ``.query()`` method will send the ``COUnter:SHOWLog?`` query.
        - Using the ``.verify(value)`` method will send the ``COUnter:SHOWLog?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``COUnter:SHOWLog value`` command.

    SCPI Syntax:
        ```
        - COUnter:SHOWLog <OFF|ON>
        - COUnter:SHOWLog?
        ```

    Info:
        - ``OFF`` disables the display of the frequency or time interval data.
        - ``ON`` enables the display of the frequency or time interval data. The default is ON.
    """


class CounterResultsValue(SCPICmdRead):
    """The ``COUnter:RESULTs:VALue`` command.

    Description:
        - Queries the measured value obtained from the Trigger Source Frequency or A-B Trigger Time
          Interval Applications. The measured quantity is value.

    Usage:
        - Using the ``.query()`` method will send the ``COUnter:RESULTs:VALue?`` query.
        - Using the ``.verify(value)`` method will send the ``COUnter:RESULTs:VALue?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - COUnter:RESULTs:VALue?
        ```
    """


class CounterResultsNumber(SCPICmdRead):
    """The ``COUnter:RESULTs:NUMber`` command.

    Description:
        - Queries the derived value obtained from the Trigger Source Frequency or A-B Trigger Time
          Interval Applications. The value is the number of acquisitions (NUMber) taken at the time
          the command was given.

    Usage:
        - Using the ``.query()`` method will send the ``COUnter:RESULTs:NUMber?`` query.
        - Using the ``.verify(value)`` method will send the ``COUnter:RESULTs:NUMber?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - COUnter:RESULTs:NUMber?
        ```
    """


class CounterResultsMinimum(SCPICmdRead):
    """The ``COUnter:RESULTs:MINimum`` command.

    Description:
        - Queries the derived value obtained from the Trigger Source Frequency or A-B Trigger Time
          Interval Applications. The minimum is a statistical value accumulated over the number of
          acquisitions ('NUMber') taken at the time the command was given.

    Usage:
        - Using the ``.query()`` method will send the ``COUnter:RESULTs:MINimum?`` query.
        - Using the ``.verify(value)`` method will send the ``COUnter:RESULTs:MINimum?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - COUnter:RESULTs:MINimum?
        ```
    """


class CounterResultsMaximum(SCPICmdRead):
    """The ``COUnter:RESULTs:MAXimum`` command.

    Description:
        - Queries the derived value obtained from the Trigger Source Frequency or A-B Trigger Time
          Interval Applications. The maximum is a statistical value accumulated over the number of
          acquisitions (NUMber) taken at the time the command was given.

    Usage:
        - Using the ``.query()`` method will send the ``COUnter:RESULTs:MAXimum?`` query.
        - Using the ``.verify(value)`` method will send the ``COUnter:RESULTs:MAXimum?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - COUnter:RESULTs:MAXimum?
        ```
    """


class CounterResultsDeviation(SCPICmdRead):
    """The ``COUnter:RESULTs:DEViation`` command.

    Description:
        - Queries the derived value obtained from the Trigger Source Frequency or A-B Trigger Time
          Interval Applications. The Deviation is a statistical value accumulated over the number of
          acquisitions (NUMber) taken at the time the command was given.

    Usage:
        - Using the ``.query()`` method will send the ``COUnter:RESULTs:DEViation?`` query.
        - Using the ``.verify(value)`` method will send the ``COUnter:RESULTs:DEViation?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - COUnter:RESULTs:DEViation?
        ```
    """


class CounterResultsAvgmean(SCPICmdRead):
    """The ``COUnter:RESULTs:AVGmean`` command.

    Description:
        - Queries the derived value obtained from the Trigger Source Frequency or A-B Trigger Time
          Interval Applications. The AVGmean is a statistical value accumulated over the number of
          acquisitions (NUMber) taken at the time the command was given.

    Usage:
        - Using the ``.query()`` method will send the ``COUnter:RESULTs:AVGmean?`` query.
        - Using the ``.verify(value)`` method will send the ``COUnter:RESULTs:AVGmean?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - COUnter:RESULTs:AVGmean?
        ```
    """


class CounterResults(SCPICmdRead):
    """The ``COUnter:RESULTs`` command.

    Description:
        - Queries the measured and derived values obtained from the Trigger Source Frequency or A-B
          Trigger Time Interval Applications. The measured quantity is value. The minimum, maximum,
          AVGmean, and Deviation are statistical values accumulated over the number of acquisitions
          (NUMber) taken at the time the command is given. All results are returned with the single
          query ``COUnter:RESULTs?`` Individual results are returned by other queries

    Usage:
        - Using the ``.query()`` method will send the ``COUnter:RESULTs?`` query.
        - Using the ``.verify(value)`` method will send the ``COUnter:RESULTs?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - COUnter:RESULTs?
        ```

    Properties:
        - ``.avgmean``: The ``COUnter:RESULTs:AVGmean`` command.
        - ``.deviation``: The ``COUnter:RESULTs:DEViation`` command.
        - ``.maximum``: The ``COUnter:RESULTs:MAXimum`` command.
        - ``.minimum``: The ``COUnter:RESULTs:MINimum`` command.
        - ``.number``: The ``COUnter:RESULTs:NUMber`` command.
        - ``.value``: The ``COUnter:RESULTs:VALue`` command.
    """

    def __init__(self, device: Optional["PIControl"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._avgmean = CounterResultsAvgmean(device, f"{self._cmd_syntax}:AVGmean")
        self._deviation = CounterResultsDeviation(device, f"{self._cmd_syntax}:DEViation")
        self._maximum = CounterResultsMaximum(device, f"{self._cmd_syntax}:MAXimum")
        self._minimum = CounterResultsMinimum(device, f"{self._cmd_syntax}:MINimum")
        self._number = CounterResultsNumber(device, f"{self._cmd_syntax}:NUMber")
        self._value = CounterResultsValue(device, f"{self._cmd_syntax}:VALue")

    @property
    def avgmean(self) -> CounterResultsAvgmean:
        """Return the ``COUnter:RESULTs:AVGmean`` command.

        Description:
            - Queries the derived value obtained from the Trigger Source Frequency or A-B Trigger
              Time Interval Applications. The AVGmean is a statistical value accumulated over the
              number of acquisitions (NUMber) taken at the time the command was given.

        Usage:
            - Using the ``.query()`` method will send the ``COUnter:RESULTs:AVGmean?`` query.
            - Using the ``.verify(value)`` method will send the ``COUnter:RESULTs:AVGmean?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - COUnter:RESULTs:AVGmean?
            ```
        """
        return self._avgmean

    @property
    def deviation(self) -> CounterResultsDeviation:
        """Return the ``COUnter:RESULTs:DEViation`` command.

        Description:
            - Queries the derived value obtained from the Trigger Source Frequency or A-B Trigger
              Time Interval Applications. The Deviation is a statistical value accumulated over the
              number of acquisitions (NUMber) taken at the time the command was given.

        Usage:
            - Using the ``.query()`` method will send the ``COUnter:RESULTs:DEViation?`` query.
            - Using the ``.verify(value)`` method will send the ``COUnter:RESULTs:DEViation?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - COUnter:RESULTs:DEViation?
            ```
        """
        return self._deviation

    @property
    def maximum(self) -> CounterResultsMaximum:
        """Return the ``COUnter:RESULTs:MAXimum`` command.

        Description:
            - Queries the derived value obtained from the Trigger Source Frequency or A-B Trigger
              Time Interval Applications. The maximum is a statistical value accumulated over the
              number of acquisitions (NUMber) taken at the time the command was given.

        Usage:
            - Using the ``.query()`` method will send the ``COUnter:RESULTs:MAXimum?`` query.
            - Using the ``.verify(value)`` method will send the ``COUnter:RESULTs:MAXimum?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - COUnter:RESULTs:MAXimum?
            ```
        """
        return self._maximum

    @property
    def minimum(self) -> CounterResultsMinimum:
        """Return the ``COUnter:RESULTs:MINimum`` command.

        Description:
            - Queries the derived value obtained from the Trigger Source Frequency or A-B Trigger
              Time Interval Applications. The minimum is a statistical value accumulated over the
              number of acquisitions ('NUMber') taken at the time the command was given.

        Usage:
            - Using the ``.query()`` method will send the ``COUnter:RESULTs:MINimum?`` query.
            - Using the ``.verify(value)`` method will send the ``COUnter:RESULTs:MINimum?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - COUnter:RESULTs:MINimum?
            ```
        """
        return self._minimum

    @property
    def number(self) -> CounterResultsNumber:
        """Return the ``COUnter:RESULTs:NUMber`` command.

        Description:
            - Queries the derived value obtained from the Trigger Source Frequency or A-B Trigger
              Time Interval Applications. The value is the number of acquisitions (NUMber) taken at
              the time the command was given.

        Usage:
            - Using the ``.query()`` method will send the ``COUnter:RESULTs:NUMber?`` query.
            - Using the ``.verify(value)`` method will send the ``COUnter:RESULTs:NUMber?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - COUnter:RESULTs:NUMber?
            ```
        """
        return self._number

    @property
    def value(self) -> CounterResultsValue:
        """Return the ``COUnter:RESULTs:VALue`` command.

        Description:
            - Queries the measured value obtained from the Trigger Source Frequency or A-B Trigger
              Time Interval Applications. The measured quantity is value.

        Usage:
            - Using the ``.query()`` method will send the ``COUnter:RESULTs:VALue?`` query.
            - Using the ``.verify(value)`` method will send the ``COUnter:RESULTs:VALue?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - COUnter:RESULTs:VALue?
            ```
        """
        return self._value


class CounterReference(SCPICmdWrite, SCPICmdRead):
    """The ``COUnter:REFerence`` command.

    Description:
        - Sets or queries the Reference (offset) value for the Trigger Source Frequency or A-B
          Trigger Time Interval applications. When the time or frequency is measured the offset is
          subtracted from the measured value before the value is displayed, used to calculate
          statistics, or logged in the table. Considered along with ``COUNT:GAIn`` <nr3>. this
          applies the linear relationship y = mx + b, where b is the Reference (or offset) and m is
          the gain (or slope). This command may be used to zero the reference value
          (``COUnter:REFerence 0``.0), or to set it to any other value. The Reference can be taken
          automatically as the last measured value using the COUnter REF command (notice no
          semicolon here), and then queried to determine its value. DPO70000SX Series only. This
          command is optional.

    Usage:
        - Using the ``.query()`` method will send the ``COUnter:REFerence?`` query.
        - Using the ``.verify(value)`` method will send the ``COUnter:REFerence?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``COUnter:REFerence value`` command.

    SCPI Syntax:
        ```
        - COUnter:REFerence <nr3>
        - COUnter:REFerence?
        ```

    Info:
        - ``<nr3>`` is the Reference (offset) value for the Trigger Source Frequency or A-B Trigger
          Time Interval applications.
    """


class CounterPrescaler(SCPICmdWrite, SCPICmdRead):
    """The ``COUnter:PREscaler`` command.

    Description:
        - Sets or queries the parameter that enables/disables the Prescaler circuits that are needed
          when the signal frequency is greater than 4 GHz. The Default is OFF. DPO70000SX Series
          only. This command is optional.

    Usage:
        - Using the ``.query()`` method will send the ``COUnter:PREscaler?`` query.
        - Using the ``.verify(value)`` method will send the ``COUnter:PREscaler?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``COUnter:PREscaler value`` command.

    SCPI Syntax:
        ```
        - COUnter:PREscaler <OFF|ON>
        - COUnter:PREscaler?
        ```

    Info:
        - ``OFF`` disables use of the prescaler.
        - ``ON`` enables use of the prescaler.
    """


class CounterLogtable(SCPICmdWrite, SCPICmdRead):
    """The ``COUnter:LOGTable`` command.

    Description:
        - Sets or queries the parameter that enables/disables the Log Table of the Trigger Source
          Frequency or A-B Trigger Time Interval Application Data. When disabled, no log is kept.
          DPO70000SX Series only. This command is optional.

    Usage:
        - Using the ``.query()`` method will send the ``COUnter:LOGTable?`` query.
        - Using the ``.verify(value)`` method will send the ``COUnter:LOGTable?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``COUnter:LOGTable value`` command.

    SCPI Syntax:
        ```
        - COUnter:LOGTable <OFF|ON>
        - COUnter:LOGTable?
        ```

    Info:
        - ``OFF`` disables use of the log table.
        - ``ON`` enables use of the log table. The default is ON.
    """


class CounterLognumber(SCPICmdWrite, SCPICmdRead):
    """The ``COUnter:LOGNumber`` command.

    Description:
        - Sets or queries the parameter that controls the schedule for updating the Log Table of the
          Trigger Source Frequency or A-B Trigger Time Interval Application Data when the
          ``COUnter:UPDate`` is NUMBER. The number of acquisitions between log updates is set to the
          number specified in the argument of this command. DPO70000SX Series only. This command is
          optional.

    Usage:
        - Using the ``.query()`` method will send the ``COUnter:LOGNumber?`` query.
        - Using the ``.verify(value)`` method will send the ``COUnter:LOGNumber?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``COUnter:LOGNumber value`` command.

    SCPI Syntax:
        ```
        - COUnter:LOGNumber <nr1>
        - COUnter:LOGNumber?
        ```

    Info:
        - ``<nr1>`` is he number of acquisitions between log updates. The default is 100
          acquisitions.
    """


class CounterLogaction(SCPICmdWrite):
    """The ``COUnter:LOGAction`` command.

    Description:
        - This command clears all data from the Log Table of the Trigger Source Frequency or A-B
          Trigger Time Interval Application. DPO70000SX Series only. This command is optional.

    Usage:
        - Using the ``.write(value)`` method will send the ``COUnter:LOGAction value`` command.

    SCPI Syntax:
        ```
        - COUnter:LOGAction <CLEAR>
        ```

    Info:
        - ``CLEAR`` empties the log table.
    """


class CounterGain(SCPICmdWrite, SCPICmdRead):
    """The ``COUnter:GAIn`` command.

    Description:
        - Sets or queries the Gain (scale) value for the Trigger Source Frequency or A-B Trigger
          Time Interval applications. When the time or frequency is measured, it is multiplied by
          the Gain before the value is displayed, used to calculate statistics, or logged in the
          table. Considered along with ``COUNT:REFerence`` <nr3>, this applies the linear
          relationship y = mx + b, where b is the Reference (or offset) and m is the gain (or
          scale). The default gain is 1.0. The gain may never be 0.0. DPO70000SX Series only. This
          command is optional.

    Usage:
        - Using the ``.query()`` method will send the ``COUnter:GAIn?`` query.
        - Using the ``.verify(value)`` method will send the ``COUnter:GAIn?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``COUnter:GAIn value`` command.

    SCPI Syntax:
        ```
        - COUnter:GAIn <nr2>
        - COUnter:GAIn?
        ```

    Info:
        - ``<nr2>`` is the Gain (scale) value for the Trigger Source Frequency or A-B Trigger Time
          Interval applications.
    """


class CounterDuration(SCPICmdWrite, SCPICmdRead):
    """The ``COUnter:DURation`` command.

    Description:
        - Sets or queries the parameter that controls the schedule for updating the Log Table of the
          Trigger Source Frequency or A-B Trigger Time Interval Application Data when the
          ``COUnter:UPDate`` is TIME. The time interval between log updates is set to the number of
          seconds specified in the argument of this command. DPO70000SX Series only. This command is
          optional.

    Usage:
        - Using the ``.query()`` method will send the ``COUnter:DURation?`` query.
        - Using the ``.verify(value)`` method will send the ``COUnter:DURation?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``COUnter:DURation value`` command.

    SCPI Syntax:
        ```
        - COUnter:DURation <nr1>
        - COUnter:DURation?
        ```

    Info:
        - ``<nr1>`` is the time interval, in seconds, between log updates. The default is 60
          seconds.
    """


#  pylint: disable=too-many-instance-attributes
class Counter(SCPICmdWrite, SCPICmdRead):
    """The ``COUnter`` command.

    Description:
        - This command is used for three separate purposes. DPO70000SX Series only. This command is
          optional. First, the ``COUnter REFerence`` command takes the reference to be the last
          measured value when the command is received. After that, the displayed values are deltas
          from the reference value. In linear terms reference is just the offset. Second, the
          ``COUnter CLEAR`` command zeroes both the measured and derived statistical values, and
          zeroes the number of acquisitions used to accumulate the statistics. Third, the
          ``COUnter RESET`` command is only needed in the Trigger Source Frequency Application to
          recalculate the A->B Sequence Trig on Nth Event value. This is only necessary if the user
          has overridden the automatically calculated Trig on Nth Event value. The application
          automatically calculates the Trig on Nth Event value to achieve an acquisition update rate
          of about 1/3 second, to both maximize the number of digits in the measured result, and
          keep the acquisition rate lively.

    Usage:
        - Using the ``.write(value)`` method will send the ``COUnter value`` command.

    SCPI Syntax:
        ```
        - COUnter <REFerence|CLEAR|RESet>
        ```

    Info:
        - ``REFerence`` takes the reference to be the last measured value when the command is
          received. After that, the displayed values are deltas from the reference value.
        - ``CLEAR`` zeroes both the measured and derived statistical values, and zeroes the number
          of acquisitions used to accumulate the statistics.
        - ``RESet`` recalculates the A->B Sequence Trig on Nth Event value.

    Properties:
        - ``.duration``: The ``COUnter:DURation`` command.
        - ``.gain``: The ``COUnter:GAIn`` command.
        - ``.logaction``: The ``COUnter:LOGAction`` command.
        - ``.lognumber``: The ``COUnter:LOGNumber`` command.
        - ``.logtable``: The ``COUnter:LOGTable`` command.
        - ``.prescaler``: The ``COUnter:PREscaler`` command.
        - ``.reference``: The ``COUnter:REFerence`` command.
        - ``.results``: The ``COUnter:RESULTs`` command.
        - ``.showlog``: The ``COUnter:SHOWLog`` command.
        - ``.showmeasurement``: The ``COUnter:SHOWMeasurement`` command.
        - ``.state``: The ``COUnter:STAte`` command.
        - ``.type``: The ``COUnter:TYPe`` command.
        - ``.update``: The ``COUnter:UPDate`` command.
        - ``.view``: The ``COUnter:VIEW`` command.
    """

    def __init__(self, device: Optional["PIControl"] = None, cmd_syntax: str = "COUnter") -> None:
        super().__init__(device, cmd_syntax)
        self._duration = CounterDuration(device, f"{self._cmd_syntax}:DURation")
        self._gain = CounterGain(device, f"{self._cmd_syntax}:GAIn")
        self._logaction = CounterLogaction(device, f"{self._cmd_syntax}:LOGAction")
        self._lognumber = CounterLognumber(device, f"{self._cmd_syntax}:LOGNumber")
        self._logtable = CounterLogtable(device, f"{self._cmd_syntax}:LOGTable")
        self._prescaler = CounterPrescaler(device, f"{self._cmd_syntax}:PREscaler")
        self._reference = CounterReference(device, f"{self._cmd_syntax}:REFerence")
        self._results = CounterResults(device, f"{self._cmd_syntax}:RESULTs")
        self._showlog = CounterShowlog(device, f"{self._cmd_syntax}:SHOWLog")
        self._showmeasurement = CounterShowmeasurement(
            device, f"{self._cmd_syntax}:SHOWMeasurement"
        )
        self._state = CounterState(device, f"{self._cmd_syntax}:STAte")
        self._type = CounterType(device, f"{self._cmd_syntax}:TYPe")
        self._update = CounterUpdate(device, f"{self._cmd_syntax}:UPDate")
        self._view = CounterView(device, f"{self._cmd_syntax}:VIEW")

    @property
    def duration(self) -> CounterDuration:
        """Return the ``COUnter:DURation`` command.

        Description:
            - Sets or queries the parameter that controls the schedule for updating the Log Table of
              the Trigger Source Frequency or A-B Trigger Time Interval Application Data when the
              ``COUnter:UPDate`` is TIME. The time interval between log updates is set to the number
              of seconds specified in the argument of this command. DPO70000SX Series only. This
              command is optional.

        Usage:
            - Using the ``.query()`` method will send the ``COUnter:DURation?`` query.
            - Using the ``.verify(value)`` method will send the ``COUnter:DURation?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``COUnter:DURation value`` command.

        SCPI Syntax:
            ```
            - COUnter:DURation <nr1>
            - COUnter:DURation?
            ```

        Info:
            - ``<nr1>`` is the time interval, in seconds, between log updates. The default is 60
              seconds.
        """
        return self._duration

    @property
    def gain(self) -> CounterGain:
        """Return the ``COUnter:GAIn`` command.

        Description:
            - Sets or queries the Gain (scale) value for the Trigger Source Frequency or A-B Trigger
              Time Interval applications. When the time or frequency is measured, it is multiplied
              by the Gain before the value is displayed, used to calculate statistics, or logged in
              the table. Considered along with ``COUNT:REFerence`` <nr3>, this applies the linear
              relationship y = mx + b, where b is the Reference (or offset) and m is the gain (or
              scale). The default gain is 1.0. The gain may never be 0.0. DPO70000SX Series only.
              This command is optional.

        Usage:
            - Using the ``.query()`` method will send the ``COUnter:GAIn?`` query.
            - Using the ``.verify(value)`` method will send the ``COUnter:GAIn?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``COUnter:GAIn value`` command.

        SCPI Syntax:
            ```
            - COUnter:GAIn <nr2>
            - COUnter:GAIn?
            ```

        Info:
            - ``<nr2>`` is the Gain (scale) value for the Trigger Source Frequency or A-B Trigger
              Time Interval applications.
        """
        return self._gain

    @property
    def logaction(self) -> CounterLogaction:
        """Return the ``COUnter:LOGAction`` command.

        Description:
            - This command clears all data from the Log Table of the Trigger Source Frequency or A-B
              Trigger Time Interval Application. DPO70000SX Series only. This command is optional.

        Usage:
            - Using the ``.write(value)`` method will send the ``COUnter:LOGAction value`` command.

        SCPI Syntax:
            ```
            - COUnter:LOGAction <CLEAR>
            ```

        Info:
            - ``CLEAR`` empties the log table.
        """
        return self._logaction

    @property
    def lognumber(self) -> CounterLognumber:
        """Return the ``COUnter:LOGNumber`` command.

        Description:
            - Sets or queries the parameter that controls the schedule for updating the Log Table of
              the Trigger Source Frequency or A-B Trigger Time Interval Application Data when the
              ``COUnter:UPDate`` is NUMBER. The number of acquisitions between log updates is set to
              the number specified in the argument of this command. DPO70000SX Series only. This
              command is optional.

        Usage:
            - Using the ``.query()`` method will send the ``COUnter:LOGNumber?`` query.
            - Using the ``.verify(value)`` method will send the ``COUnter:LOGNumber?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``COUnter:LOGNumber value`` command.

        SCPI Syntax:
            ```
            - COUnter:LOGNumber <nr1>
            - COUnter:LOGNumber?
            ```

        Info:
            - ``<nr1>`` is he number of acquisitions between log updates. The default is 100
              acquisitions.
        """
        return self._lognumber

    @property
    def logtable(self) -> CounterLogtable:
        """Return the ``COUnter:LOGTable`` command.

        Description:
            - Sets or queries the parameter that enables/disables the Log Table of the Trigger
              Source Frequency or A-B Trigger Time Interval Application Data. When disabled, no log
              is kept. DPO70000SX Series only. This command is optional.

        Usage:
            - Using the ``.query()`` method will send the ``COUnter:LOGTable?`` query.
            - Using the ``.verify(value)`` method will send the ``COUnter:LOGTable?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``COUnter:LOGTable value`` command.

        SCPI Syntax:
            ```
            - COUnter:LOGTable <OFF|ON>
            - COUnter:LOGTable?
            ```

        Info:
            - ``OFF`` disables use of the log table.
            - ``ON`` enables use of the log table. The default is ON.
        """
        return self._logtable

    @property
    def prescaler(self) -> CounterPrescaler:
        """Return the ``COUnter:PREscaler`` command.

        Description:
            - Sets or queries the parameter that enables/disables the Prescaler circuits that are
              needed when the signal frequency is greater than 4 GHz. The Default is OFF. DPO70000SX
              Series only. This command is optional.

        Usage:
            - Using the ``.query()`` method will send the ``COUnter:PREscaler?`` query.
            - Using the ``.verify(value)`` method will send the ``COUnter:PREscaler?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``COUnter:PREscaler value`` command.

        SCPI Syntax:
            ```
            - COUnter:PREscaler <OFF|ON>
            - COUnter:PREscaler?
            ```

        Info:
            - ``OFF`` disables use of the prescaler.
            - ``ON`` enables use of the prescaler.
        """
        return self._prescaler

    @property
    def reference(self) -> CounterReference:
        """Return the ``COUnter:REFerence`` command.

        Description:
            - Sets or queries the Reference (offset) value for the Trigger Source Frequency or A-B
              Trigger Time Interval applications. When the time or frequency is measured the offset
              is subtracted from the measured value before the value is displayed, used to calculate
              statistics, or logged in the table. Considered along with ``COUNT:GAIn`` <nr3>. this
              applies the linear relationship y = mx + b, where b is the Reference (or offset) and m
              is the gain (or slope). This command may be used to zero the reference value
              (``COUnter:REFerence 0``.0), or to set it to any other value. The Reference can be
              taken automatically as the last measured value using the COUnter REF command (notice
              no semicolon here), and then queried to determine its value. DPO70000SX Series only.
              This command is optional.

        Usage:
            - Using the ``.query()`` method will send the ``COUnter:REFerence?`` query.
            - Using the ``.verify(value)`` method will send the ``COUnter:REFerence?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``COUnter:REFerence value`` command.

        SCPI Syntax:
            ```
            - COUnter:REFerence <nr3>
            - COUnter:REFerence?
            ```

        Info:
            - ``<nr3>`` is the Reference (offset) value for the Trigger Source Frequency or A-B
              Trigger Time Interval applications.
        """
        return self._reference

    @property
    def results(self) -> CounterResults:
        """Return the ``COUnter:RESULTs`` command.

        Description:
            - Queries the measured and derived values obtained from the Trigger Source Frequency or
              A-B Trigger Time Interval Applications. The measured quantity is value. The minimum,
              maximum, AVGmean, and Deviation are statistical values accumulated over the number of
              acquisitions (NUMber) taken at the time the command is given. All results are returned
              with the single query ``COUnter:RESULTs?`` Individual results are returned by other
              queries

        Usage:
            - Using the ``.query()`` method will send the ``COUnter:RESULTs?`` query.
            - Using the ``.verify(value)`` method will send the ``COUnter:RESULTs?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - COUnter:RESULTs?
            ```

        Sub-properties:
            - ``.avgmean``: The ``COUnter:RESULTs:AVGmean`` command.
            - ``.deviation``: The ``COUnter:RESULTs:DEViation`` command.
            - ``.maximum``: The ``COUnter:RESULTs:MAXimum`` command.
            - ``.minimum``: The ``COUnter:RESULTs:MINimum`` command.
            - ``.number``: The ``COUnter:RESULTs:NUMber`` command.
            - ``.value``: The ``COUnter:RESULTs:VALue`` command.
        """
        return self._results

    @property
    def showlog(self) -> CounterShowlog:
        """Return the ``COUnter:SHOWLog`` command.

        Description:
            - Sets or queries the parameter that controls the display of the Trigger Source
              Frequency or A-B Trigger Time Interval Application Data. The display of the data in
              the graticule area may be turned OFF or back ON as desired. A possible reason for
              turning it OFF is when the Log Table is enabled to record the data. DPO70000SX Series
              only. This command is optional.

        Usage:
            - Using the ``.query()`` method will send the ``COUnter:SHOWLog?`` query.
            - Using the ``.verify(value)`` method will send the ``COUnter:SHOWLog?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``COUnter:SHOWLog value`` command.

        SCPI Syntax:
            ```
            - COUnter:SHOWLog <OFF|ON>
            - COUnter:SHOWLog?
            ```

        Info:
            - ``OFF`` disables the display of the frequency or time interval data.
            - ``ON`` enables the display of the frequency or time interval data. The default is ON.
        """
        return self._showlog

    @property
    def showmeasurement(self) -> CounterShowmeasurement:
        """Return the ``COUnter:SHOWMeasurement`` command.

        Description:
            - Sets or queries the parameter that controls the display of the Trigger Source
              Frequency or A-B Trigger Time Interval Application Data. The display of the data in
              the graticule area may be turned OFF or back ON as desired. A possible reason for
              turning it OFF is when the Log Table is enabled to record the data. DPO70000SX Series
              only. This command is optional.

        Usage:
            - Using the ``.query()`` method will send the ``COUnter:SHOWMeasurement?`` query.
            - Using the ``.verify(value)`` method will send the ``COUnter:SHOWMeasurement?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``COUnter:SHOWMeasurement value``
              command.

        SCPI Syntax:
            ```
            - COUnter:SHOWMeasurement <OFF|ON>
            - COUnter:SHOWMeasurement?
            ```

        Info:
            - ``OFF`` disables the display of the frequency or time interval data.
            - ``ON`` enables the display of the frequency or time interval data. The default is ON.
        """
        return self._showmeasurement

    @property
    def state(self) -> CounterState:
        """Return the ``COUnter:STAte`` command.

        Description:
            - Enables the Trigger Source Frequency or A-B Trigger Time Interval application,
              depending on the value of the ``COUnter:TYPe`` parameter. This command sets or queries
              the Counter State enumerated value (OFF/ON) used to enable/disable the display of
              frequency or time information in the graticule area or in the Results Table. The
              Trigger Source Frequency Application uses trigger hardware, rather than the acquired
              waveform, to measure the frequency of the signal on the trigger source to a very high
              precision. Best results are obtained when the External Reference is tied between the
              instrument and the source of the trigger signal. As a result of using the trigger
              hardware, the measurement update rate is very fast, 1/3 second by default. An A-Event
              Edge to B-Event Edge A->B Sequence must be used. Statistics are derived from the
              measured values: Minimum, Maximum, Average (mean), and Standard Deviation. The number
              of measurements made so far is also included. Note that long record lengths are NOT
              needed for this application, because the measurements are made from the trigger
              hardware, not from the acquired waveform. A Reference value may be taken by command or
              entered manually. When the Reference is non-zero, the measured value and the derived
              statistics are shown or queried as delta values from the Reference. The A-B Trigger
              Time Interval Application also uses trigger hardware to measure the time between the
              A-Event and the B-Event in an A-B Sequence trigger. The A- and B-Events may be any
              combination of trigger types: Edge, Glitch, Width, etc. A Reference value may be taken
              by command or entered manually. When the Reference value is non-zero, the measured
              value is displayed or queried as a delta from this Reference. For example, if the
              ``COUnter:TYPe`` parameter is set to FREQuency, the ``COUnter:STAte ON`` command
              causes the display of the frequency of the triggered channel signal, which is assumed
              to be an infinitely repeating clock. Along with the frequency value, the minimum,
              maximum, average (mean), and standard deviation of the frequency are also displayed.
              Finally, a reference value is displayed. The reference is 0.0 by default, but may be
              captured as the last measured value at any time using the COUnter REF command. When
              the reference value is nonzero, the other values are deltas from the reference value.
              The displayed information is typical of products generally referred to by the industry
              as 'requency Counters. At the user's discretion, the ``COUnter:VIEW`` <PERiod |
              FREQuency> command can be used to view the information as the Frequency (Hz) or Period
              (seconds). The default view is Frequency. As a second example, if the ``COUnter:TYPe``
              parameter is set to TIME, the ``COUnter:STAte ON`` command causes the display of the
              Time interval from the Trigger A-Event to the Trigger B-Event. These events can be
              from any trigger types in an A->B Trigger Sequence. In this case only the Time and the
              Reference value are displayed. The COUnter REF command takes the Reference value as
              before, but in this case the units relate to Time (seconds). DPO70000SX Series only.
              This command is optional.

        Usage:
            - Using the ``.query()`` method will send the ``COUnter:STAte?`` query.
            - Using the ``.verify(value)`` method will send the ``COUnter:STAte?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``COUnter:STAte value`` command.

        SCPI Syntax:
            ```
            - COUnter:STAte {ON|OFF|<NR1>}
            - COUnter:STAte?
            ```

        Info:
            - ``<NR1>`` , if 0 disables the counter function, if 1 enables the counter function.
            - ``OFF`` disables the counter function. The default value is OFF.
            - ``ON`` enables the counter function.
        """
        return self._state

    @property
    def type(self) -> CounterType:
        """Return the ``COUnter:TYPe`` command.

        Description:
            - Sets or queries the Counter Application type as Trigger Source Frequency or A-B
              Trigger Time Interval. The Trigger Source Frequency Application is commonly referred
              to as Frequency Counting, and measures the frequency of the trigger source over a
              large number of events to obtain a very accurate result. The A-B Trigger Time Interval
              Application measures the A-Event to B-Event in an A->B Sequence Trigger, over a single
              A->B Trigger Sequence. DPO70000SX Series only. This command is optional.

        Usage:
            - Using the ``.query()`` method will send the ``COUnter:TYPe?`` query.
            - Using the ``.verify(value)`` method will send the ``COUnter:TYPe?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``COUnter:TYPe value`` command.

        SCPI Syntax:
            ```
            - COUnter:TYPe <TIME|FREQuency>
            - COUnter:TYPe?
            ```

        Info:
            - ``TIME`` sets the application type to frequency.
            - ``FREQuency`` sets the application type to time interval. The default value is
              Frequency.
        """
        return self._type

    @property
    def update(self) -> CounterUpdate:
        """Return the ``COUnter:UPDate`` command.

        Description:
            - Sets or queries the parameter that controls the schedule for updating the Log Table of
              the Trigger Source Frequency or A-B Trigger Time Interval Application Data. The
              schedule for logging the data may be set to occur automatically, after a given time
              interval has elapsed, or after a given number of acquisitions have occurred. The
              default is AUTO. DPO70000SX Series only. This command is optional.

        Usage:
            - Using the ``.query()`` method will send the ``COUnter:UPDate?`` query.
            - Using the ``.verify(value)`` method will send the ``COUnter:UPDate?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``COUnter:UPDate value`` command.

        SCPI Syntax:
            ```
            - COUnter:UPDate <AUTO|TIME|NUMBER>
            - COUnter:UPDate?
            ```

        Info:
            - ``AUTO`` sets the data logging to automatic.
            - ``TIME`` sets the data logging to occur after a given time interval has elapsed.
            - ``NUMBER`` sets the data logging to occur after a given number of acquisitions have
              occurred.
        """
        return self._update

    @property
    def view(self) -> CounterView:
        """Return the ``COUnter:VIEW`` command.

        Description:
            - Sets or queries the View of the Counter Trigger Source Frequency Application Data.
              When the ``COUnter:TYPe`` is FREQuency, the data may be viewed as Frequency (Hz) or
              Time (seconds). This command does not apply to the A-B Time Interval Application
              (``COUnter:TYPE TIME``). DPO70000SX Series only. This command is optional.

        Usage:
            - Using the ``.query()`` method will send the ``COUnter:VIEW?`` query.
            - Using the ``.verify(value)`` method will send the ``COUnter:VIEW?`` query and raise an
              AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``COUnter:VIEW value`` command.

        SCPI Syntax:
            ```
            - COUnter:VIEW <FREQuency|PERiod>
            - COUnter:VIEW?
            ```

        Info:
            - ``FREQuency`` displays the count in Hertz.
            - ``PERiod`` displays the count in seconds.
        """
        return self._view



"""The autosaveuitimeout commands module.

These commands are used in the following models:
DPO7AX, LPD6, MSO4, MSO4B, MSO5, MSO5B, MSO5LP, MSO6, MSO6B

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - AUTOSAVEUITIMEOUT <NR1>
    - AUTOSAVEUITIMEOUT?
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import SCPICmdRead, SCPICmdWrite

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.pi_control import PIControl


class Autosaveuitimeout(SCPICmdWrite, SCPICmdRead):
    """The ``AUTOSAVEUITIMEOUT`` command.

    Description:
        - This command sets or queries the idle time from the user interface before auto-save
          occurs.

    Usage:
        - Using the ``.query()`` method will send the ``AUTOSAVEUITIMEOUT?`` query.
        - Using the ``.verify(value)`` method will send the ``AUTOSAVEUITIMEOUT?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``AUTOSAVEUITIMEOUT value`` command.

    SCPI Syntax:
        ```
        - AUTOSAVEUITIMEOUT <NR1>
        - AUTOSAVEUITIMEOUT?
        ```

    Info:
        - ``<NR1>`` sets the timeout time.
    """

    def __init__(
        self, device: Optional["PIControl"] = None, cmd_syntax: str = "AUTOSAVEUITIMEOUT"
    ) -> None:
        super().__init__(device, cmd_syntax)



"""AWG5K device driver module."""

from types import MappingProxyType
from typing import Dict, Optional, Tuple

from tm_devices.commands import AWG5KMixin
from tm_devices.drivers.awgs.awg import (
    AWG,
    AWGSourceChannel,
    AWGSourceDeviceConstants,
    ParameterBounds,
)
from tm_devices.drivers.device import family_base_class
from tm_devices.helpers import ReadOnlyCachedProperty as cached_property  # noqa: N813
from tm_devices.helpers.enums import SignalGeneratorOutputPathsBase


@family_base_class
class AWG5K(AWG5KMixin, AWG):
    """AWG5K device driver."""

    _DEVICE_CONSTANTS = AWGSourceDeviceConstants(
        memory_page_size=1,
        memory_max_record_length=16200000,
        memory_min_record_length=1,
    )

    ################################################################################################
    # Magic Methods
    ################################################################################################

    ################################################################################################
    # Properties
    ################################################################################################
    @cached_property
    def source_channel(self) -> "MappingProxyType[str, AWGSourceChannel]":
        """Mapping of channel names to AWG5KSourceChannel objects."""
        channel_map: Dict[str, AWG5KSourceChannel] = {}
        for channel_name in self.all_channel_names_list:
            channel_map[channel_name] = AWG5KSourceChannel(self, channel_name)
        return MappingProxyType(channel_map)

    ################################################################################################
    # Private Methods
    ################################################################################################
    def _get_series_specific_constraints(
        self,
        output_signal_path: Optional[SignalGeneratorOutputPathsBase],
    ) -> Tuple[ParameterBounds, ParameterBounds, ParameterBounds]:
        """Get constraints which are dependent on the model series and parameters.

        Args:
            output_signal_path: The signal path that the output is taking.

        Returns:
            Ranges for amplitude, offset, and sample rate.
        """
        if not output_signal_path:
            output_signal_path = self.OutputSignalPath.DCA

        # if DIR output path is disconnected
        if output_signal_path == self.OutputSignalPath.DCA:
            amplitude_range = ParameterBounds(lower=20.0e-3, upper=4.5)
            offset_range = ParameterBounds(lower=-2.25, upper=2.25)
        # otherwise the DIR output path is connected
        else:
            amplitude_range = ParameterBounds(lower=20.0e-3, upper=0.6)
            offset_range = ParameterBounds(lower=-0.0, upper=0.0)

        # AWG(Arbitrary Waveform Generator)5(Series)0x(.6 + .6x GS/s)x(x Channels)z(Model)
        # AWG5012B means the sample rate max is 1.2 GHz, a AWG5002B would have a 600 MHz max
        sample_rate_range = ParameterBounds(
            lower=10.0e6, upper=600.0e6 + (600.0e6 * int(self.model[5]))
        )
        return amplitude_range, offset_range, sample_rate_range


class AWG5KSourceChannel(AWGSourceChannel):
    """AWG5K signal source channel composite."""

    _awg: AWG5K

    ################################################################################################
    # Magic Methods
    ################################################################################################

    ################################################################################################
    # Public Methods
    ################################################################################################
    def set_offset(self, value: float, absolute_tolerance: float = 0) -> None:
        """Set the offset on the source channel.

        Args:
            value: The offset value to set.
            absolute_tolerance: The acceptable difference between two floating point values.
        """
        if not float(self._awg.query(f"AWGCONTROL:DOUTPUT{self.num}:STATE?")):
            # Can only set offset when then output state is 0.
            self._awg.set_if_needed(
                f"{self.name}:VOLTAGE:OFFSET",
                value,
                tolerance=absolute_tolerance,
            )
        elif value:
            # No error is raised when 0 is the offset value and the output signal path
            # is in a state where offset cannot be set.
            offset_error = (
                f"The offset can only be set on {self._awg.model} with an output signal path of "
                f"{self._awg.OutputSignalPath.DCA.value} "
                f"(AWGCONTROL:DOUTPUT{self.num}:STATE set to 0)."
            )
            raise ValueError(offset_error)

    def set_output_signal_path(
        self, value: Optional[SignalGeneratorOutputPathsBase] = None
    ) -> None:
        """Set the output signal path on the source channel.

        Args:
            value: The output signal path.
        """
        if not value or value == self._awg.OutputSignalPath.DCA:
            # Translate DCA to output state of 0.
            output_state = 0
        elif value == self._awg.OutputSignalPath.DIR:
            # Translate DIR to output state of 1.
            output_state = 1
        else:
            output_signal_path_error = (
                f"{value.value} is an invalid output signal path for {self._awg.model}."
            )
            raise ValueError(output_signal_path_error)
        self._awg.set_if_needed(f"AWGCONTROL:DOUTPUT{self.num}:STATE", output_state)



# pyright: reportConstantRedefinition=none
"""The os commands module.

These commands are used in the following models:
SS3706A

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - os.time()
    ```
"""

from typing import Optional, TYPE_CHECKING

from ..helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.driver_mixins.device_control.tsp_control import TSPControl


class Os(BaseTSPCmd):
    """The ``os`` command tree.

    Properties and methods:
        - ``.time()``: The ``os.time()`` function.
    """

    def __init__(self, device: Optional["TSPControl"] = None, cmd_syntax: str = "os") -> None:
        super().__init__(device, cmd_syntax)

    def time(self, timespec: Optional[str] = None) -> str:
        """Run the ``os.time()`` function.

        Description:
            - This function generates a time value in UTC time.

        TSP Syntax:
            ```
            - os.time()
            ```

        Args:
            timespec (optional): The date and time (year, month, day, hour, and minute).

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(str(x) for x in (timespec,) if x is not None)
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.time({function_args}))"
            )
        except AttributeError as error:
            msg = f"No TSPControl object was provided, unable to run the ``{self._cmd_syntax}.time()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error



"""AWG5200 device driver module."""

from pathlib import Path
from types import MappingProxyType
from typing import cast, Dict, Literal, Optional, Tuple

from tm_devices.commands import AWG5200Mixin
from tm_devices.drivers.awgs.awg import (
    AWG,
    AWGSourceChannel,
    AWGSourceDeviceConstants,
    ParameterBounds,
)
from tm_devices.drivers.device import family_base_class
from tm_devices.helpers import ReadOnlyCachedProperty as cached_property  # noqa: N813
from tm_devices.helpers import (
    SASSetWaveformFileTypes,
)
from tm_devices.helpers.enums import (
    SignalGeneratorFunctionsAWG,
    SignalGeneratorOutputPaths5200,
    SignalGeneratorOutputPathsBase,
)


@family_base_class
class AWG5200(AWG5200Mixin, AWG):
    """AWG5200 device driver."""

    OutputSignalPath = SignalGeneratorOutputPaths5200

    _DEVICE_CONSTANTS = AWGSourceDeviceConstants(
        memory_page_size=1,
        memory_max_record_length=16200000,
        memory_min_record_length=1,
    )
    sample_waveform_set_file = (
        "C:\\Program Files\\Tektronix\\AWG5200\\Samples\\AWG5k7k Predefined Waveforms.awgx"
    )

    ################################################################################################
    # Magic Methods
    ################################################################################################

    ################################################################################################
    # Properties
    ################################################################################################
    @cached_property
    def source_channel(self) -> "MappingProxyType[str, AWGSourceChannel]":
        """Mapping of channel names to AWG5200SourceChannel objects."""
        channel_map: Dict[str, AWG5200SourceChannel] = {}
        for channel_name in self.all_channel_names_list:
            channel_map[channel_name] = AWG5200SourceChannel(self, channel_name)
        return MappingProxyType(channel_map)

    ################################################################################################
    # Public Methods
    ################################################################################################
    def load_waveform_set(self, waveform_set_file: Optional[str] = None) -> None:
        """Load a waveform set into the memory of the AWG.

        Arguments:
            waveform_set_file: The waveform set file to load
                (The default is defined in the ``sample_wave_file`` attribute).
        """
        self._load_waveform_or_set(waveform_set_file=waveform_set_file, waveform_name=None)

    def load_waveform_from_set(
        self,
        waveform_name: str,
        waveform_set_file: Optional[str] = None,
    ) -> None:
        """Load in a specific waveform from a waveform set into the memory of the AWG.

        Arguments:
            waveform_name: The waveform name to load from the waveform set file.
            waveform_set_file: The waveform set file to load
                (The default is defined in the ``sample_wave_file`` attribute).
        """
        self._load_waveform_or_set(waveform_set_file=waveform_set_file, waveform_name=waveform_name)

    def generate_function(  # noqa: PLR0913
        self,
        frequency: float,
        function: SignalGeneratorFunctionsAWG,
        amplitude: float,
        offset: float,
        channel: str = "all",
        output_signal_path: Optional[SignalGeneratorOutputPathsBase] = None,
        termination: Literal["FIFTY", "HIGHZ"] = "FIFTY",
        duty_cycle: float = 50.0,
        polarity: Literal["NORMAL", "INVERTED"] = "NORMAL",
        symmetry: float = 50.0,
    ) -> None:
        """Generate a predefined waveform given the following parameters.

        Args:
            frequency: The frequency of the waveform to generate.
            function: The waveform shape to generate.
            amplitude: The amplitude of the signal to generate.
            offset: The offset of the signal to generate.
            channel: The channel name to output the signal from, or 'all'.
            output_signal_path: The output signal path of the specified channel.
            termination: The impedance this device's ``channel`` expects to see at the received end.
            duty_cycle: The duty cycle percentage within [10.0, 90.0].
            polarity: The polarity to set the signal to.
            symmetry: The symmetry to set the signal to, only applicable to certain functions.
        """
        predefined_name, needed_sample_rate = self._get_predefined_waveform_name(
            frequency, function, output_signal_path, symmetry
        )
        self.generate_waveform(
            needed_sample_rate=needed_sample_rate,
            waveform_name=predefined_name,
            amplitude=amplitude,
            offset=offset,
            channel=channel,
            output_signal_path=output_signal_path,
            termination=termination,
            duty_cycle=duty_cycle,
            polarity=polarity,
            symmetry=symmetry,
        )

    def generate_waveform(  # noqa: PLR0913
        self,
        needed_sample_rate: float,
        waveform_name: str,
        amplitude: float,
        offset: float,
        channel: str = "all",
        output_signal_path: Optional[SignalGeneratorOutputPathsBase] = None,
        termination: Literal["FIFTY", "HIGHZ"] = "FIFTY",  # noqa: ARG002
        duty_cycle: float = 50.0,  # noqa: ARG002
        polarity: Literal["NORMAL", "INVERTED"] = "NORMAL",  # noqa: ARG002
        symmetry: float = 50.0,  # noqa: ARG002
    ) -> None:
        """Generate a waveform given the following parameters.

        Args:
            needed_sample_rate: The required sample rate.
            waveform_name: The name of the waveform to generate.
            amplitude: The amplitude of the signal to generate.
            offset: The offset of the signal to generate.
            channel: The channel name to output the signal from, or 'all'.
            output_signal_path: The output signal path of the specified channel.
            termination: The impedance this device's ``channel`` expects to see at the received end.
            duty_cycle: The duty cycle percentage within [10.0, 90.0].
            polarity: The polarity to set the signal to.
            symmetry: The symmetry to set the signal to, only applicable to certain functions.
        """
        # If the waveform is not in the waveform list, load in the waveform set defined at
        # self.sample_waveform_set_file
        if waveform_name not in self.query("WLISt:LIST?", allow_empty=True).replace('"', "").split(
            ","
        ):
            self.load_waveform_set()
        for channel_name in self._validate_channels(channel):
            self.set_sample_rate(needed_sample_rate)
            source_channel = cast("AWG5200SourceChannel", self.source_channel[channel_name])
            # turn channel off
            self.set_and_check(f"OUTPUT{source_channel.num}:STATE", "0")
            source_channel.set_waveform_properties(
                output_signal_path=output_signal_path,
                waveform_name=waveform_name,
                amplitude=amplitude,
                offset=offset,
            )
            self.set_if_needed(f"OUTPUT{source_channel.num}:STATE", "1")
        # this is an overlapping command
        self.write("AWGCONTROL:RUN")
        self.ieee_cmds.opc()
        # we expect no errors
        self.expect_esr(0)

    def set_sample_rate(self, value: float, absolute_tolerance: Optional[float] = None) -> None:
        """Set the rate at which samples are generated/transmitted.

        Args:
            value: The sample rate to set.
            absolute_tolerance: The acceptable difference between two floating point values.
                                Default value is 0.1% of the provided value.
        """
        if absolute_tolerance is None:
            # Default the absolute tolerance to 0.1% of the provided frequency value
            # due to 32 bit rounding.
            absolute_tolerance = value * 0.001
        # This is an overlapping command for the AWG5200, and will overlap the
        # next command and/or overlap the previous if it is still running.
        self.set_if_needed("CLOCK:SRATE", value, verify_value=False, tolerance=absolute_tolerance)

    ################################################################################################
    # Private Methods
    ################################################################################################
    def _get_series_specific_constraints(
        self,
        output_signal_path: Optional[SignalGeneratorOutputPathsBase],
    ) -> Tuple[ParameterBounds, ParameterBounds, ParameterBounds]:
        """Get constraints which are dependent on the model series and parameters.

        Args:
            output_signal_path: The signal path that the output is taking.

        Returns:
            Ranges for amplitude, offset, and sample rate.
        """
        if not output_signal_path:
            output_signal_path = self.OutputSignalPath.DCHB
        # Direct Current High Bandwidth with the DC options has 1.5 V amplitude
        if "DC" in self.opt_string and output_signal_path == self.OutputSignalPath.DCHB:
            amplitude_range = ParameterBounds(lower=25.0e-3, upper=1.5)
        # Direct Current High Voltage path connected has an even higher amplitude, 5 V
        elif output_signal_path == self.OutputSignalPath.DCHV:
            amplitude_range = ParameterBounds(lower=10.0e-3, upper=5.0)
        # Else, the upper bound is 750 mV
        else:
            amplitude_range = ParameterBounds(lower=25.0e-3, upper=750.0e-3)

        offset_range = ParameterBounds(lower=-2.0, upper=2.0)
        # option is the sample rate in hundreds of Mega Hertz
        max_sample_rate = 25.0 if "25" in self.opt_string else 50.0
        # option 50 would have 5.0 GHz, option 2.5 would have 2.5 GHz
        sample_rate_range = ParameterBounds(lower=300.0, upper=max_sample_rate * 100.0e6)

        return amplitude_range, offset_range, sample_rate_range

    def _load_waveform_or_set(
        self,
        waveform_set_file: Optional[str] = None,
        waveform_name: Optional[str] = None,
    ) -> None:
        """Load in a waveform set or a specific waveform from a waveform set into memory.

        Arguments:
            waveform_set_file: The waveform set file to load.
                (The default is defined in the ``sample_wave_file`` attribute).
            waveform_name: The waveform name to load from the waveform set file.
        """
        # This function is identical to the one in the AWG70KA.
        if not waveform_set_file:
            waveform_set_file = self.sample_waveform_set_file
        waveform_file_type = Path(waveform_set_file).suffix.lower()
        try:
            SASSetWaveformFileTypes(waveform_file_type)
            if not waveform_name:
                self.write(f'MMEMORY:OPEN:SASSET "{waveform_set_file}"', opc=True)
            else:
                self.write(
                    f'MMEMORY:OPEN:SASSET:WAVEFORM "{waveform_set_file}", "{waveform_name}"',
                    opc=True,
                )
        except ValueError as err:
            waveform_file_type_error = (
                f"{waveform_file_type} is an invalid waveform file extension."
            )
            raise ValueError(waveform_file_type_error) from err

        self.expect_esr(0)


class AWG5200SourceChannel(AWGSourceChannel):
    """AWG5200 signal source channel composite."""

    _awg: AWG5200

    ################################################################################################
    # Magic Methods
    ################################################################################################

    ################################################################################################
    # Public Methods
    ################################################################################################
    def load_waveform(self, waveform_name: str) -> None:
        """Load in a waveform from the waveform list to the source channel.

        Args:
            waveform_name: The name of the waveform to load.
        """
        self._awg.ieee_cmds.opc()
        self._awg.set_if_needed(f"{self.name}:WAVEFORM", f'"{waveform_name}"', allow_empty=True)

    def set_offset(self, value: float, absolute_tolerance: float = 0) -> None:
        """Set the offset on the source channel.

        Args:
            value: The offset value to set.
            absolute_tolerance: The acceptable difference between two floating point values.
        """
        self._awg.set_if_needed(
            f"{self.name}:VOLTAGE:OFFSET",
            value,
            tolerance=absolute_tolerance,
        )

    def set_output_signal_path(
        self, value: Optional[SignalGeneratorOutputPathsBase] = None
    ) -> None:
        """Set the output signal path on the source channel.

        Args:
            value: The output signal path.
        """
        if not value:
            value = self._awg.OutputSignalPath.DCHB
        if value not in self._awg.OutputSignalPath:
            output_signal_path_error = (
                f"{value.value} is an invalid output signal path for {self._awg.model}."
            )
            raise ValueError(output_signal_path_error)
        self._awg.set_if_needed(f"OUTPUT{self.num}:PATH", value.value)
